//===========================================================================//
//                                                                           //
//  Copyright (C) 2004 - 2019                                                //
//  IDS Imaging Development Systems GmbH                                     //
//  Dimbacherstr. 6-8                                                        //
//  D-74182 Obersulm-Willsbach                                               //
//                                                                           //
//  The information in this document is subject to change without            //
//  notice and should not be construed as a commitment by IDS Imaging GmbH.  //
//  IDS Imaging GmbH does not assume any responsibility for any errors       //
//  that may appear in this document.                                        //
//                                                                           //
//  This document, or source code, is provided solely as an example          //
//  of how to utilize IDS software libraries in a sample application.        //
//  IDS Imaging GmbH does not assume any responsibility for the use or       //
//  reliability of any portion of this document or the described software.   //
//                                                                           //
//  General permission to copy or modify, but not for profit, is hereby      //
//  granted,  provided that the above copyright notice is included and       //
//  reference made to the fact that reproduction privileges were granted     //
//  by IDS Imaging GmbH.                                                     //
//                                                                           //
//  IDS cannot assume any responsibility for the use or misuse of any        //
//  portion of this software for other than its intended diagnostic purpose  //
//  in calibrating and testing IDS manufactured cameras and software.        //
//                                                                           //
//===========================================================================//

#include "stdafx.h"

#include "IdsSherpa.h"

#include <peak\peak.hpp>
#include <peak_ipl\peak_ipl.hpp>
#include <peak\converters\peak_buffer_converter_ipl.hpp>    


IdsSherpa::IdsSherpa(HWND hWnd)
{
	m_hWnd = hWnd;
	m_bmpInfo = NULL;
    m_threadRunning = FALSE;

    m_pixelFormat = peak::ipl::PixelFormatName::BGRa8;

    m_mapPixelFormat[peak::ipl::PixelFormatName::Mono8]         = "Mono8";
    m_mapPixelFormat[peak::ipl::PixelFormatName::Mono10]        = "Mono10";
    m_mapPixelFormat[peak::ipl::PixelFormatName::Mono10p]       = "Mono10p";
    m_mapPixelFormat[peak::ipl::PixelFormatName::Mono12]        = "Mono12";
    m_mapPixelFormat[peak::ipl::PixelFormatName::Mono12p]       = "Mono12p";
    m_mapPixelFormat[peak::ipl::PixelFormatName::BayerRG8]      = "BayerRG8";
    m_mapPixelFormat[peak::ipl::PixelFormatName::BayerRG10]     = "BayerRG10";
    m_mapPixelFormat[peak::ipl::PixelFormatName::BayerRG10p]    = "BayerRG10p";
    m_mapPixelFormat[peak::ipl::PixelFormatName::BayerRG12]     = "BayerRG12";
    m_mapPixelFormat[peak::ipl::PixelFormatName::BayerRG12p]    = "BayerRG12p";
    m_mapPixelFormat[peak::ipl::PixelFormatName::RGB8]          = "RGB8";
    m_mapPixelFormat[peak::ipl::PixelFormatName::BGR8]          = "BGR8";
    m_mapPixelFormat[peak::ipl::PixelFormatName::BGRa8]         = "BGRa8";
    m_mapPixelFormat[peak::ipl::PixelFormatName::RGBa8]         = "RGBa8";
    m_mapPixelFormat[peak::ipl::PixelFormatName::RGB10]         = "RGB10";
    m_mapPixelFormat[peak::ipl::PixelFormatName::BGR10]         = "BGR10";
    m_mapPixelFormat[peak::ipl::PixelFormatName::RGBa10]        = "RGBa10";
    m_mapPixelFormat[peak::ipl::PixelFormatName::BGRa10]        = "BGRa10";
    m_mapPixelFormat[peak::ipl::PixelFormatName::RGB10p32]      = "RGB10p32";
    m_mapPixelFormat[peak::ipl::PixelFormatName::BGR10p32]      = "BGR10p32";
    m_mapPixelFormat[peak::ipl::PixelFormatName::RGB12]         = "RGB12";
    m_mapPixelFormat[peak::ipl::PixelFormatName::BGR12]         = "BGR12";
    m_mapPixelFormat[peak::ipl::PixelFormatName::RGBa12]        = "RGBa12";
    m_mapPixelFormat[peak::ipl::PixelFormatName::BGRa12]        = "BGRa12";

    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::BayerRG8] = peak::ipl::PixelFormatName::BayerRG8;
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::Mono8] = peak::ipl::PixelFormatName::BayerRG8;
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::BGR8] = peak::ipl::PixelFormatName::BayerRG8;
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::BGRa8] = peak::ipl::PixelFormatName::BayerRG8;
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::RGB8] = peak::ipl::PixelFormatName::BayerRG8;
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::RGBa8] = peak::ipl::PixelFormatName::BayerRG8;
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::BayerRG10] = peak::ipl::PixelFormatName::BayerRG10;
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::Mono10] = peak::ipl::PixelFormatName::BayerRG10;
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::BGR10] = peak::ipl::PixelFormatName::BayerRG10; // cannot be displayed without reducing bits
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::BGRa10] = peak::ipl::PixelFormatName::BayerRG10; // cannot be displayed without reducing bits
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::RGB10] = peak::ipl::PixelFormatName::BayerRG10; // cannot be displayed without reducing bits
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::RGBa10] = peak::ipl::PixelFormatName::BayerRG10; // cannot be displayed without reducing bits
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::RGB10p32] = peak::ipl::PixelFormatName::BayerRG10;
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::BayerRG12] = peak::ipl::PixelFormatName::BayerRG12;
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::Mono12] = peak::ipl::PixelFormatName::BayerRG12;
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::BGR12] = peak::ipl::PixelFormatName::BayerRG12; // cannot be displayed without reducing bits
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::BGRa12] = peak::ipl::PixelFormatName::BayerRG12; // cannot be displayed without reducing bits
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::RGB12] = peak::ipl::PixelFormatName::BayerRG12; // cannot be displayed without reducing bits
    m_mapChoosePixelFormatColor[peak::ipl::PixelFormatName::RGBa12] = peak::ipl::PixelFormatName::BayerRG12; // cannot be displayed without reducing bits

    m_mapChoosePixelFormatMono[peak::ipl::PixelFormatName::Mono8] = peak::ipl::PixelFormatName::Mono8;
    m_mapChoosePixelFormatMono[peak::ipl::PixelFormatName::BGR8] = peak::ipl::PixelFormatName::Mono8;
    m_mapChoosePixelFormatMono[peak::ipl::PixelFormatName::BGRa8] = peak::ipl::PixelFormatName::Mono8;
    m_mapChoosePixelFormatMono[peak::ipl::PixelFormatName::RGB8] = peak::ipl::PixelFormatName::Mono8;
    m_mapChoosePixelFormatMono[peak::ipl::PixelFormatName::RGBa8] = peak::ipl::PixelFormatName::Mono8;
    m_mapChoosePixelFormatMono[peak::ipl::PixelFormatName::Mono10] = peak::ipl::PixelFormatName::Mono10;
    m_mapChoosePixelFormatMono[peak::ipl::PixelFormatName::BGR10] = peak::ipl::PixelFormatName::Mono10; // cannot be displayed without reducing bits
    m_mapChoosePixelFormatMono[peak::ipl::PixelFormatName::BGRa10] = peak::ipl::PixelFormatName::Mono10; // cannot be displayed without reducing bits
    m_mapChoosePixelFormatMono[peak::ipl::PixelFormatName::RGB10] = peak::ipl::PixelFormatName::Mono10; // cannot be displayed without reducing bits
    m_mapChoosePixelFormatMono[peak::ipl::PixelFormatName::RGBa10] = peak::ipl::PixelFormatName::Mono10; // cannot be displayed without reducing bits
    m_mapChoosePixelFormatMono[peak::ipl::PixelFormatName::Mono12] = peak::ipl::PixelFormatName::Mono12;
    m_mapChoosePixelFormatMono[peak::ipl::PixelFormatName::BGR12] = peak::ipl::PixelFormatName::Mono12; // cannot be displayed without reducing bits
    m_mapChoosePixelFormatMono[peak::ipl::PixelFormatName::BGRa12] = peak::ipl::PixelFormatName::Mono12; // cannot be displayed without reducing bits
    m_mapChoosePixelFormatMono[peak::ipl::PixelFormatName::RGB12] = peak::ipl::PixelFormatName::Mono12; // cannot be displayed without reducing bits
    m_mapChoosePixelFormatMono[peak::ipl::PixelFormatName::RGBa12] = peak::ipl::PixelFormatName::Mono12; // cannot be displayed without reducing bits

    m_renderType = DISPLAY_FIT_TO_WINDOW;

    // Initialize the peak library for all further accesses. Without this call all other peak function calls will fail
    // You should close the library at the end of the program over the function peak::Library::Close()
    peak::Library::Initialize();
}


IdsSherpa::~IdsSherpa()
{
	StopCapture();

	if (m_bmpInfo)
	{
		delete[] m_bmpInfo;
		m_bmpInfo = NULL;
	}

    peak::Library::Close();
}


bool IdsSherpa::SetWindowHandle(HWND hWnd)
{
    m_hWnd = hWnd;
    return true;
}


bool IdsSherpa::InitCamera(std::string serialNumber)
{
    try
    {
        auto& deviceManager = peak::DeviceManager::Instance();
        deviceManager.Update();

        // Open first available camera
        if ("" == serialNumber)
        {
            if (!deviceManager.Devices().empty())
            {
                if (peak::core::DeviceAccessStatus::ReadWrite == deviceManager.Devices().at(0)->AccessStatus())
                {
                    // Open camera. The function returns an object for all further accesses to the camera
                    m_device = deviceManager.Devices().at(0)->OpenDevice(peak::core::DeviceAccessType::Control);
                }
            }
        }
        // Open camera by serial number
        else
        {
            for (const auto& deviceDescriptor : deviceManager.Devices())
            {
                if (deviceDescriptor->SerialNumber() == serialNumber)
                {
                    if (peak::core::DeviceAccessStatus::ReadWrite == deviceDescriptor->AccessStatus())
                    {
                        m_device = deviceDescriptor->OpenDevice(peak::core::DeviceAccessType::Control);
                        break;
                    }
                }
            }
        }

        if (m_device)
        {
            // Get standard data stream
            m_dataStream = m_device->DataStreams().at(0)->OpenDataStream();

            // Get remote device nodemap for all accesses to the genicam nodemap tree
            m_nodemapRemoteDevice = m_device->RemoteDevice()->NodeMaps().at(0);

            // Check if camera is mono or color
            m_color = false;
            for (const auto& pixelformat : m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("PixelFormat")->Entries())
            {
                // If the camera supports bayer, it is a color camera
                if ("BayerRG8" == pixelformat->SymbolicValue())
                {
                    m_color = true;
                    break;
                }
            }

            InitDisplay();

            return true;
        }
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::ExitCamera()
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        m_nodemapRemoteDevice = nullptr;
        m_dataStream = nullptr;
        m_device = nullptr;

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetNumberOfCameras(int& number)
{
    try
    {
        auto& deviceManager = peak::DeviceManager::Instance();
        deviceManager.Update();

        number = static_cast<int>(deviceManager.Devices().size());

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetColorModeSource(peak::ipl::PixelFormatName& colorMode)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        std::string pixelFormat = m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("PixelFormat")->CurrentEntry()->SymbolicValue();

        for (std::map<peak::ipl::PixelFormatName, std::string>::const_iterator it = m_mapPixelFormat.begin(); it != m_mapPixelFormat.end(); ++it)
        {
            if (it->second == pixelFormat)
            {
                colorMode = it->first;
                return true;
            }
        }
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::SetColorModeSource(peak::ipl::PixelFormatName colorMode)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("PixelFormat")->SetCurrentEntry(m_mapPixelFormat[colorMode]);

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetColorModeDest(peak::ipl::PixelFormatName& colorMode)
{
    if (!m_device)
    {
        return false;
    }

    colorMode = m_pixelFormat;

    return true;
}


bool IdsSherpa::SetColorModeDest(peak::ipl::PixelFormatName colorMode)
{
    if (!m_device)
    {
        return false;
    }

    m_pixelFormat = colorMode;

    if (false == InitDisplay())
    {
        return false;
    }

    return true;
}


bool IdsSherpa::GetColorMode(peak::ipl::PixelFormatName& colorMode)
{
    return GetColorModeDest(colorMode);
}


bool IdsSherpa::SetColorMode(peak::ipl::PixelFormatName colorMode)
{   
    peak::ipl::PixelFormatName pixelFormatSource = peak::ipl::PixelFormatName::Invalid;

    if (IsColorCamera())
    {
        pixelFormatSource = m_mapChoosePixelFormatColor[colorMode];
    }
    else
    {
        pixelFormatSource = m_mapChoosePixelFormatMono[colorMode];
    }

    if (peak::ipl::PixelFormatName::Invalid == pixelFormatSource)
    {
        return false;
    }

    if (false == SetColorModeDest(colorMode))
    {
        return false;
    }

    if (false == SetColorModeSource(pixelFormatSource))
    {
        return false;
    }

    return true;
}


bool IdsSherpa::GetBitsPerPixel(peak::ipl::PixelFormatName colorMode, int& bitsPerPixel)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        bitsPerPixel = static_cast<int>(peak::ipl::PixelFormat(colorMode).NumSignificantBitsPerPixel());

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetImageAoiMin(int& x, int& y, int& width, int& height)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        x = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("OffsetX")->Minimum());
        y = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("OffsetY")->Minimum());
        width = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("Width")->Minimum());
        height = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("Height")->Minimum());

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetImageAoiMax(int& x, int& y, int& width, int& height)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        x = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("OffsetX")->Maximum());
        y = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("OffsetY")->Maximum());
        width = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("Width")->Maximum());
        height = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("Height")->Maximum());

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetImageAoi(int& x, int& y, int& width, int& height)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        x = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("OffsetX")->Value());
        y = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("OffsetY")->Value());
        width = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("Width")->Value());
        height = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("Height")->Value());

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::SetImageAoi(int x, int y, int width, int height)
{
    if (!m_device)
    {
        return false;
    }

    int x_min = 0;
    int y_min = 0;
    int width_min = 0;
    int height_min = 0;

    if (!GetImageAoiMin(x_min, y_min, width_min, height_min))
    {
        return false;
    }

    try
    {
        // Set the minimum AOI: After that all size restrictions allow setting any AOI possible
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("OffsetX")->SetValue(x_min);
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("OffsetY")->SetValue(y_min);
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("Width")->SetValue(width_min);
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("Height")->SetValue(height_min);
    }
    catch (std::exception&)
    {
        return false;
    }

    int x_max = 0;
    int y_max = 0;
    int width_max = 0;
    int height_max = 0;

    if (!GetImageAoiMax(x_max, y_max, width_max, height_max))
    {
        return false;
    }

    // Some checks
    if (width < width_min)
    {
        width = width_min;
    }

    if (height < height_min)
    {
        height = height_min;
    }

    if ((x + width) > width_max)
    {
        width = width_max - x;
    }

    if (y + height > height_max)
    {
        height = height_max - y;
    }

    try
    {
        // Set the final AOI
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("OffsetX")->SetValue(x);
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("OffsetY")->SetValue(y);
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("Width")->SetValue(width);
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("Height")->SetValue(height);
    }
    catch (std::exception&)
    {
        return false;
    }

    return true;
}


bool IdsSherpa::GetCameraInfo(CAMERA_INFORMATION& info)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        info.serialNumber = m_device->SerialNumber();
        info.manufacturer = m_device->VendorName();
        info.cameraType = m_nodemapRemoteDevice->FindNode<peak::core::nodes::StringNode>("DeviceFamilyName")->Value();
        info.modelName = m_device->ModelName();
        info.maxSensorWidth = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("SensorWidth")->Value());
        info.maxSensorHeight = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("SensorHeight")->Value());

        std::string value = m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("PixelColorFilter")->CurrentEntry()->StringValue();
        if ("None" == value)
        {
            info.bayerPattern = false;
            info.upperLeftBayerPixel = "";
        }
        else if ("BayerRG" == value)
        {
            info.bayerPattern = true;
            info.upperLeftBayerPixel = "red";
        }
        else if (("BayerGR" == value) || ("BayerGB" == value))
        {
            info.bayerPattern = true;
            info.upperLeftBayerPixel = "green";
        }
        else if ("BayerBG" == value)
        {
            info.bayerPattern = true;
            info.upperLeftBayerPixel = "blue";
        }
       
        info.globalShutter = ("Global" == m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("SensorShutterMode")->CurrentEntry()->StringValue());
    }
    catch (std::exception&)
    {
        return false;
    }

    return true;
}


bool IdsSherpa::AllocImageBuffersAndCreateSequence(int numberBuffers)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        if (0 == numberBuffers)
        {
            // Get the minimum number of buffers that must be announced
            numberBuffers = static_cast<int>(m_dataStream->NumBuffersAnnouncedMinRequired());
        }

        // Get the payload size for correct buffer allocation
        int64_t payloadSize = m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("PayloadSize")->Value();

        // Allocate "numberBuffers" buffers (RAW image) and queue them
        for (int count = 0; count < numberBuffers; ++count)
        {
            auto buffer = m_dataStream->AllocAndAnnounceBuffer(static_cast<size_t>(payloadSize), nullptr);
            m_dataStream->QueueBuffer(buffer);
        }

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::FreeImageBuffersAndClearSequence()
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        if (m_dataStream)
        {
            // Flush queue and prepare all buffers for revoking
            m_dataStream->Flush(peak::core::DataStreamFlushMode::DiscardAll);

            // Revoke all buffers
            for (const auto& buffer : m_dataStream->AnnouncedBuffers())
            {
                m_dataStream->RevokeBuffer(buffer);
            }
        }

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::SetRenderType(int renderType)
{
    if (!m_device)
    {
        return false;
    }

    m_renderType = renderType;

    return true;
}


bool IdsSherpa::InitEvents()
{
    if (!m_device)
    {
        return false;
    }

    // Nothing to do. Peak enables events automatically
    return true;
}


bool IdsSherpa::ExitEvents()
{
    if (!m_device)
    {
        return false;
    }

    // Nothing to do. Peak enables events automatically
    return true;
}


bool IdsSherpa::StartCapture()
{
    if (!m_device)
    {
        return false;
    }

	if (m_threadRunning)
	{
		return false;
	}

	m_threadCapture = AfxBeginThread(CaptureWorker, this);

    try
    {
        // Start continuous acquisition
        m_dataStream->StartAcquisition(peak::core::AcquisitionStartMode::Default, peak::core::INFINITE_NUMBER);
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::CommandNode>("AcquisitionStart")->Execute();
    
        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::StopCapture()
{
    if (!m_device)
    {
        return false;
    }

	if (!m_threadRunning)
	{
		return false;
	}

	m_stopThread = TRUE;

	while (m_threadRunning)
	{
		Sleep(10);
	}

    try
    {
	    // Stop continuous acquisition
	    m_nodemapRemoteDevice->FindNode<peak::core::nodes::CommandNode>("AcquisitionStop")->Execute();
	    m_dataStream->StopAcquisition(peak::core::AcquisitionStopMode::Default);

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::IsCapturing()
{
    if (!m_device)
    {
        return false;
    }

    if (m_threadRunning)
    {
        return true;
    }
    else
    {
        return false;
    }
}


bool IdsSherpa::GetFramerateMin(double& min)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        min = m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("AcquisitionFrameRate")->Minimum();
       
        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetFramerateMax(double& max)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        max = m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("AcquisitionFrameRate")->Maximum();

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetFramerate(double& framerate)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        framerate = m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("AcquisitionFrameRate")->Value();

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::SetFramerate(double framerate)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("AcquisitionFrameRate")->SetValue(framerate);

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetExposureTimeMin_ms(double& min)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        min = m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("ExposureTime")->Minimum() / 1000.0;

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetExposureTimeMax_ms(double& max)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        max = m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("ExposureTime")->Maximum() / 1000.0;

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetExposureTime_ms(double& exposure)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        exposure = m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("ExposureTime")->Value() / 1000.0;

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::SetExposureTime_ms(double exposure)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("ExposureTime")->SetValue(exposure * 1000.0);

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetTriggerMode(int& mode)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        std::string triggerSelector = m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("TriggerSelector")->CurrentEntry()->SymbolicValue();
        std::string triggerMode = m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("TriggerMode")->CurrentEntry()->SymbolicValue();
        std::string triggerActivation = m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("TriggerActivation")->CurrentEntry()->SymbolicValue();
        std::string triggerSource = m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("TriggerSource")->CurrentEntry()->SymbolicValue();

        if ("ExposureStart" != triggerSelector)
        {
            return false;
        }

        if ("Off" == triggerMode)
        {
            mode = TRIGGER_OFF;
        }
        else if (("On" == triggerMode) && ("FallingEdge" == triggerActivation) && ("Line0" == triggerSource))
        {
            mode = TRIGGER_HI_LO;
        }
        else if (("On" == triggerMode) && ("RisingEdge" == triggerActivation) && ("Line0" == triggerSource))
        {
            mode = TRIGGER_LO_HI;
        }
        else if (("On" == triggerMode) && ("Software" == triggerSource))
        {
            mode = TRIGGER_SOFTWARE;
        }

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::SetTriggerMode(int mode)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        // Set trigger selector to "Exposure start"
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("TriggerSelector")->SetCurrentEntry("ExposureStart");

        if (TRIGGER_OFF == mode)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("TriggerMode")->SetCurrentEntry("Off");
        }
        else
        {
            if (TRIGGER_HI_LO == mode)
            {
                m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("TriggerSource")->SetCurrentEntry("Line0");
                m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("TriggerActivation")->SetCurrentEntry("FallingEdge");
            }
            else if (TRIGGER_LO_HI == mode)
            {
                m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("TriggerSource")->SetCurrentEntry("Line0");
                m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("TriggerActivation")->SetCurrentEntry("RisingEdge");
            }
            else if (TRIGGER_SOFTWARE == mode)
            {
                m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("TriggerSource")->SetCurrentEntry("Software");
            }
            
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("TriggerMode")->SetCurrentEntry("On");
        }

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::ForceTrigger()
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::CommandNode>("TriggerSoftware")->Execute();

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::SetTriggerTimeout_ms(int timeout)
{
    // Nothing to do because this feature works different with peak
    return true;
}


bool IdsSherpa::GetTriggerDelayMin_us(int& min)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        min = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("TriggerDelay")->Minimum());

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetTriggerDelayMax_us(int& max)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        max = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("TriggerDelay")->Maximum());

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetTriggerDelayInc_us(int& inc)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        inc = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("TriggerDelay")->Increment());

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetTriggerDelay_us(int& delay)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        delay = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("TriggerDelay")->Value());

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::SetTriggerDelay_us(int delay)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("TriggerDelay")->SetValue(static_cast<double>(delay));

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetGainMin(int type, double& min)
{
    return GainHelper(type, 0, min);
}


bool IdsSherpa::GetGainMax(int type, double& max)
{
    return GainHelper(type, 1, max);
}


bool IdsSherpa::GetGain(int type, double& gain)
{
    return GainHelper(type, 2, gain);
}


bool IdsSherpa::SetGain(int type, double gain)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        if (GAIN_MASTER_ANALOG == type)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("GainSelector")->SetCurrentEntry("AnalogAll");
        }
        else if (GAIN_MASTER_DIGITAL == type)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("GainSelector")->SetCurrentEntry("DigitalAll");
        }
        else if (GAIN_RED == type)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("GainSelector")->SetCurrentEntry("DigitalRed");
        }
        else if (GAIN_GREEN == type)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("GainSelector")->SetCurrentEntry("DigitalGreen");
        }
        else if (GAIN_BLUE == type)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("GainSelector")->SetCurrentEntry("DigitalBlue");
        }

        m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("Gain")->SetValue(gain);
        
        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


/****************************************************************************/
/* Auto brightness - exposure                                               */
/****************************************************************************/
bool IdsSherpa::GetAutoExposure(int& autoExposure)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        std::string value = m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("ExposureAuto")->CurrentEntry()->SymbolicValue();

        if ("Off" == value)
        {
            autoExposure = AUTO_EXPOSURE_OFF;
        }
        else if ("Once" == value)
        {
            autoExposure = AUTO_EXPOSURE_ONCE;
        }
        else if ("Continuous" == value)
        {
            autoExposure = AUTO_EXPOSURE_CONTINUOUS;
        }

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::SetAutoExposure(int autoExposure)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        if (AUTO_EXPOSURE_OFF == autoExposure)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("ExposureAuto")->SetCurrentEntry("Off");
        }
        else if (AUTO_EXPOSURE_ONCE == autoExposure)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("ExposureAuto")->SetCurrentEntry("Once");
        }
        else if (AUTO_EXPOSURE_CONTINUOUS == autoExposure)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("ExposureAuto")->SetCurrentEntry("Continuous");
        }

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


/****************************************************************************/
/* Auto brightness - gain                                                   */
/****************************************************************************/
bool IdsSherpa::GetAutoGain(int& autoGain)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        std::string value = m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("GainAuto")->CurrentEntry()->SymbolicValue();

        if ("Off" == value)
        {
            autoGain = AUTO_GAIN_OFF;
        }
        else if ("Once" == value)
        {
            autoGain = AUTO_GAIN_ONCE;
        }
        else if ("Continuous" == value)
        {
            autoGain = AUTO_GAIN_CONTINUOUS;
        }

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::SetAutoGain(int autoGain)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        if (AUTO_GAIN_OFF == autoGain)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("GainAuto")->SetCurrentEntry("Off");
        }
        else if (AUTO_GAIN_ONCE == autoGain)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("GainAuto")->SetCurrentEntry("Once");
        }
        else if (AUTO_GAIN_CONTINUOUS == autoGain)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("GainAuto")->SetCurrentEntry("Continuous");
        }

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


/****************************************************************************/
/* Auto brightness - percentile                                             */
/****************************************************************************/
bool IdsSherpa::GetBrightnessAutoPercentileMin(double& min)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        min = m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("BrightnessAutoPercentile")->Minimum();

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetBrightnessAutoPercentileMax(double& max)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        max = m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("BrightnessAutoPercentile")->Maximum();

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetBrightnessAutoPercentile(double& value)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        value = m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("BrightnessAutoPercentile")->Value();

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::SetBrightnessAutoPercentile(double value)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("BrightnessAutoPercentile")->SetValue(value);

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


/****************************************************************************/
/* Auto brightness - target                                                 */
/****************************************************************************/
bool IdsSherpa::GetBrightnessAutoTargetMin(int& min)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        min = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("BrightnessAutoTarget")->Minimum());

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetBrightnessAutoTargetMax(int& max)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        max = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("BrightnessAutoTarget")->Maximum());

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetBrightnessAutoTarget(int& value)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        value = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("BrightnessAutoTarget")->Value());

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::SetBrightnessAutoTarget(int value)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("BrightnessAutoTarget")->SetValue(static_cast<int64_t>(value));

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetBrightnessAutoTargetToleranceMin(int& min)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        min = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("BrightnessAutoTargetTolerance")->Minimum());

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetBrightnessAutoTargetToleranceMax(int& max)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        max = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("BrightnessAutoTargetTolerance")->Maximum());

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetBrightnessAutoTargetTolerance(int& value)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        value = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("BrightnessAutoTargetTolerance")->Value());

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::SetBrightnessAutoTargetTolerance(int value)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("BrightnessAutoTargetTolerance")->SetValue(static_cast<int64_t>(value));

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::SaveParametersInCameraMemory(int userSet)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        if (USER_SET_0 == userSet)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("UserSetSelector")->SetCurrentEntry("UserSet0");
        }
        else if (USER_SET_1 == userSet)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("UserSetSelector")->SetCurrentEntry("UserSet1");
        }

        m_nodemapRemoteDevice->FindNode<peak::core::nodes::CommandNode>("UserSetSave")->Execute();

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::LoadParametersFromCameraMemory(int userSet)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        if (USER_SET_0 == userSet)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("UserSetSelector")->SetCurrentEntry("UserSet0");
        }
        else if (USER_SET_1 == userSet)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("UserSetSelector")->SetCurrentEntry("UserSet1");
        }
        
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::CommandNode>("UserSetLoad")->Execute();

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::GetSubsamplingFactor(int& x, int& y)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("DecimationSelector")->SetCurrentEntry("Region0");
        x = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("DecimationHorizontal")->Value());
        y = static_cast<int>(m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("DecimationVertical")->Value());

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::SetSubsamplingFactor(int x, int y)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("DecimationSelector")->SetCurrentEntry("Region0");
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("DecimationHorizontal")->SetValue(static_cast<int64_t>(x));
        m_nodemapRemoteDevice->FindNode<peak::core::nodes::IntegerNode>("DecimationVertical")->SetValue(static_cast<int64_t>(y));

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


/**************************************************************************************/
/* Internal functions                                                                 */
/**************************************************************************************/
bool IdsSherpa::GainHelper(int type, int min_max_val, double& value)
{
    if (!m_device)
    {
        return false;
    }

    try
    {
        if (GAIN_MASTER_ANALOG == type)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("GainSelector")->SetCurrentEntry("AnalogAll");
        }
        else if (GAIN_MASTER_DIGITAL == type)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("GainSelector")->SetCurrentEntry("DigitalAll");
        }
        else if (GAIN_RED == type)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("GainSelector")->SetCurrentEntry("DigitalRed");
        }
        else if (GAIN_GREEN == type)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("GainSelector")->SetCurrentEntry("DigitalGreen");
        }
        else if (GAIN_BLUE == type)
        {
            m_nodemapRemoteDevice->FindNode<peak::core::nodes::EnumerationNode>("GainSelector")->SetCurrentEntry("DigitalBlue");
        }

        if (0 == min_max_val)
        {
            value = m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("Gain")->Minimum();
        }
        else if (1 == min_max_val)
        {
            value = m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("Gain")->Maximum();
        }
        else if (2 == min_max_val)
        {
            value = m_nodemapRemoteDevice->FindNode<peak::core::nodes::FloatNode>("Gain")->Value();
        }

        return true;
    }
    catch (std::exception&)
    {
    }

    return false;
}


bool IdsSherpa::IsColorCamera()
{
    return m_color;
}


bool IdsSherpa::InitDisplay()
{
    int bitsPerPixel = 32;

    if (false == GetBitsPerPixel(m_pixelFormat, bitsPerPixel))
    {
        return false;
    }

    INT nSize = sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * (bitsPerPixel <= 8 ? 256 : 3);

    if (m_bmpInfo)
    {
        delete[] m_bmpInfo;
        m_bmpInfo = NULL;
    }

    m_bmpInfo = new char[nSize];
    ZeroMemory(m_bmpInfo, sizeof(m_bmpInfo));

    BITMAPINFOHEADER* dibhdr = (BITMAPINFOHEADER*)m_bmpInfo;

    dibhdr->biSize = sizeof(BITMAPINFOHEADER);
    dibhdr->biPlanes = 1;
    dibhdr->biBitCount = bitsPerPixel;
    dibhdr->biCompression = BI_BITFIELDS;
    dibhdr->biXPelsPerMeter = 0;
    dibhdr->biYPelsPerMeter = 0;
    dibhdr->biClrUsed = 256;
    dibhdr->biClrImportant = 256;

    int iR = 0;
    int iG = 1;
    int iB = 2;

    long* pl = (long*)((BITMAPINFO*)dibhdr)->bmiColors;


    switch (m_pixelFormat)
    {
        case peak::ipl::PixelFormatName::BGR8:
        case peak::ipl::PixelFormatName::RGB8:
        case peak::ipl::PixelFormatName::BGR10: // Must be reduced to 8 bits
        case peak::ipl::PixelFormatName::RGB10: // Must be reduced to 8 bits
        case peak::ipl::PixelFormatName::BGR12: // Must be reduced to 8 bits
        case peak::ipl::PixelFormatName::RGB12: // Must be reduced to 8 bits
        {
            dibhdr->biCompression = BI_RGB;
        }
        break;

        case peak::ipl::PixelFormatName::Mono8:
        case peak::ipl::PixelFormatName::BayerRG8:
        {
            dibhdr->biCompression = BI_RGB;

            for (int i = 0; i < 256; i++)
            {
                ((BITMAPINFO*)dibhdr)->bmiColors[i].rgbBlue = i;
                ((BITMAPINFO*)dibhdr)->bmiColors[i].rgbRed = i;
                ((BITMAPINFO*)dibhdr)->bmiColors[i].rgbGreen = i;
            }
        }
        break;

        case peak::ipl::PixelFormatName::Mono10:
        case peak::ipl::PixelFormatName::BayerRG10:
        {
            dibhdr->biBitCount = 16;

            pl[iR] = 0x000003FF;
            pl[iG] = 0x000003FF;
            pl[iB] = 0x000003FF;
        }
        break;

        case peak::ipl::PixelFormatName::Mono12:
        case peak::ipl::PixelFormatName::BayerRG12:
        {
            dibhdr->biBitCount = 16;

            pl[iR] = 0x00000FFF;
            pl[iG] = 0x00000FFF;
            pl[iB] = 0x00000FFF;
        }
        break;

        case peak::ipl::PixelFormatName::BGR10p32:
        {
            dibhdr->biBitCount = 32;

            pl[iR] = 0x3FF00000;
            pl[iG] = 0x000FFC00;
            pl[iB] = 0x000003FF;
        }
        break;

        case peak::ipl::PixelFormatName::RGB10p32:
        {
            dibhdr->biBitCount = 32;

            pl[iB] = 0x3FF00000;
            pl[iG] = 0x000FFC00;
            pl[iR] = 0x000003FF;
        }
        break;

        case peak::ipl::PixelFormatName::BGRa8:
        case peak::ipl::PixelFormatName::RGBa8:
        case peak::ipl::PixelFormatName::BGRa10: // Must be reduced to 8 bits
        case peak::ipl::PixelFormatName::RGBa10: // Must be reduced to 8 bits
        case peak::ipl::PixelFormatName::BGRa12: // Must be reduced to 8 bits
        case peak::ipl::PixelFormatName::RGBa12: // Must be reduced to 8 bits
        default:
        {
            pl[iR] = 0x00FF0000;
            pl[iG] = 0x0000FF00;
            pl[iB] = 0x000000FF;
        }
        break;
    }

    return true;
}


UINT IdsSherpa::CaptureWorker(LPVOID param)
{
    IdsSherpa* parent = static_cast<IdsSherpa*>(param);

    if (NULL == parent)
    {
        return FALSE;
    }

    if (!parent->m_device)
    {
        return FALSE;
    }

    if (NULL == parent->m_hWnd)
    {
        return FALSE;
    }

    // Image size
    int x = 0;
    int y = 0;
    int imageWidth = 0;
    int imageHeight = 0;
    if (!parent->GetImageAoi(x, y, imageWidth, imageHeight))
    {
        return FALSE;
    }

    CRect rcWindow;
    ::GetClientRect(parent->m_hWnd, rcWindow);

    int m_sizeDestX = rcWindow.Width();
    int m_sizeDestY = rcWindow.Height();

    if (DISPLAY_NORMAL == parent->m_renderType)
    {
        m_sizeDestX = imageWidth;
        m_sizeDestY = imageHeight;
    }

    BITMAPINFOHEADER* dibhdr = (BITMAPINFOHEADER*)parent->m_bmpInfo;
    dibhdr->biWidth = imageWidth;
    dibhdr->biHeight = -imageHeight;
    dibhdr->biSizeImage = (DWORD)(imageWidth * imageHeight);
    
    HDC hdc = ::GetDC(parent->m_hWnd);
    if (!hdc)
    {
        return false;
    }

    ::SetStretchBltMode(hdc, COLORONCOLOR);

    parent->m_stopThread = FALSE;
    parent->m_threadRunning = TRUE;

    while (TRUE == parent->m_threadRunning)
    {
        try
        {
            // Wait 5 seconds for an image from the camera
            auto imageBuffer = parent->m_dataStream->WaitForFinishedBuffer(5000);

            // Convert image
            auto imageProcessed = peak::BufferTo<peak::ipl::Image>(imageBuffer).ConvertTo(parent->m_pixelFormat);

            // Requeue buffer
            parent->m_dataStream->QueueBuffer(imageBuffer);

            int ret = ::StretchDIBits(hdc,
                                      0,
                                      0,
                                      m_sizeDestX,
                                      m_sizeDestY,
                                      0,
                                      0,
                                      imageWidth,
                                      imageHeight,
                                      reinterpret_cast<char*>(imageProcessed.Data()),
                                      (BITMAPINFO*)parent->m_bmpInfo,
                                      DIB_RGB_COLORS,
                                      SRCCOPY
                                     );
        }
        catch (const std::exception&)
        {
        }

        if (TRUE == parent->m_stopThread)
        {
            parent->m_threadRunning = FALSE;
        }
    }

    ReleaseDC(parent->m_hWnd, hdc);

    return true;
}
