//===========================================================================//
//                                                                           //
//  Copyright (C) 2004 - 2019                                                //
//  IDS Imaging Development Systems GmbH                                     //
//  Dimbacherstr. 6-8                                                        //
//  D-74182 Obersulm-Willsbach                                               //
//                                                                           //
//  The information in this document is subject to change without            //
//  notice and should not be construed as a commitment by IDS Imaging GmbH.  //
//  IDS Imaging GmbH does not assume any responsibility for any errors       //
//  that may appear in this document.                                        //
//                                                                           //
//  This document, or source code, is provided solely as an example          //
//  of how to utilize IDS software libraries in a sample application.        //
//  IDS Imaging GmbH does not assume any responsibility for the use or       //
//  reliability of any portion of this document or the described software.   //
//                                                                           //
//  General permission to copy or modify, but not for profit, is hereby      //
//  granted,  provided that the above copyright notice is included and       //
//  reference made to the fact that reproduction privileges were granted     //
//  by IDS Imaging GmbH.                                                     //
//                                                                           //
//  IDS cannot assume any responsibility for the use or misuse of any        //
//  portion of this software for other than its intended diagnostic purpose  //
//  in calibrating and testing IDS manufactured cameras and software.        //
//                                                                           //
//===========================================================================//

#include "stdafx.h"
#include "IdsSherpa_uEye.h"
#include "IdsSherpa_uEye_dlg.h"

extern IdsSherpa_uEye theApp;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


IdsSherpa_uEye_dlg::IdsSherpa_uEye_dlg(CWnd* pParent /*=NULL*/)
: CDialog(IdsSherpa_uEye_dlg::IDD, pParent)
{
    m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}


void IdsSherpa_uEye_dlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);

    DDX_Control(pDX, IDC_SLIDER_FRAMERATE_UEYE, m_sliderFramerate_uEye);
    DDX_Control(pDX, IDC_LABEL_FRAMERATE_MIN_UEYE, m_labelFramerateMin_uEye);
    DDX_Control(pDX, IDC_LABEL_FRAMERATE_MAX_UEYE, m_labelFramerateMax_uEye);
    DDX_Control(pDX, IDC_LABEL_FRAMERATE_UEYE, m_labelFramerate_uEye);

    DDX_Control(pDX, IDC_SLIDER_EXPOSURE_UEYE, m_sliderExposure_uEye);
    DDX_Control(pDX, IDC_LABEL_EXPOSURE_MIN_UEYE, m_labelExposureMin_uEye);
    DDX_Control(pDX, IDC_LABEL_EXPOSURE_MAX_UEYE, m_labelExposureMax_uEye);
    DDX_Control(pDX, IDC_LABEL_EXPOSURE_UEYE, m_labelExposure_uEye);

    DDX_Control(pDX, IDC_COMBO_TRIGGER_MODE_UEYE, m_comboTriggerMode_uEye);

    DDX_Control(pDX, IDC_SLIDER_FRAMERATE_PEAK, m_sliderFramerate_peak);
    DDX_Control(pDX, IDC_LABEL_FRAMERATE_MIN_PEAK, m_labelFramerateMin_peak);
    DDX_Control(pDX, IDC_LABEL_FRAMERATE_MAX_PEAK, m_labelFramerateMax_peak);
    DDX_Control(pDX, IDC_LABEL_FRAMERATE_PEAK, m_labelFramerate_peak);

    DDX_Control(pDX, IDC_SLIDER_EXPOSURE_PEAK, m_sliderExposure_peak);
    DDX_Control(pDX, IDC_LABEL_EXPOSURE_MIN_PEAK, m_labelExposureMin_peak);
    DDX_Control(pDX, IDC_LABEL_EXPOSURE_MAX_PEAK, m_labelExposureMax_peak);
    DDX_Control(pDX, IDC_LABEL_EXPOSURE_PEAK, m_labelExposure_peak);

    DDX_Control(pDX, IDC_COMBO_TRIGGER_MODE_PEAK, m_comboTriggerMode_peak);
}


BEGIN_MESSAGE_MAP(IdsSherpa_uEye_dlg, CDialog)
    ON_WM_SYSCOMMAND()
    ON_WM_PAINT()
    ON_WM_QUERYDRAGICON()
    ON_BN_CLICKED(IDC_BUTTON_START_UEYE, OnButtonStart_uEye)
	ON_BN_CLICKED(IDC_BUTTON_STOP_UEYE, OnButtonStop_uEye)
	ON_BN_CLICKED(IDC_BUTTON_START_PEAK, OnButtonStart_peak)
	ON_BN_CLICKED(IDC_BUTTON_STOP_PEAK, OnButtonStop_peak)
    ON_BN_CLICKED(IDC_BUTTON_EXIT, OnButtonExit)
    ON_WM_CLOSE()
    ON_WM_HSCROLL()
    ON_CBN_SELCHANGE(IDC_COMBO_TRIGGER_MODE_UEYE, OnComboTriggerMode_uEye)
    ON_CBN_SELCHANGE(IDC_COMBO_TRIGGER_MODE_PEAK, OnComboTriggerMode_peak)
    ON_BN_CLICKED(IDC_BUTTON_FORCE_TRIGGER_UEYE, OnBnClickedButtonForceTriggerUeye)
    ON_BN_CLICKED(IDC_BUTTON_FORCE_TRIGGER_PEAK, OnBnClickedButtonForceTriggerPeak)
END_MESSAGE_MAP()


BOOL IdsSherpa_uEye_dlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    SetIcon(m_hIcon, TRUE); // Set big icon
    SetIcon(m_hIcon, FALSE); // Set small icon


    /********************************************************************************************************/
    /*                                                                                                      */
    /*       uEye SDK                                                                                       */ 
    /*                                                                                                      */    
    /********************************************************************************************************/

    // Open uEye camera. Pass the window handle as argument
    m_uEyeCamera = std::make_shared<IdsuEye>(GetDlgItem(IDC_DISPLAY)->m_hWnd);

    // Get number of connected uEye cameras
    int numberOfCameras = 0;
    bool ret = m_uEyeCamera->GetNumberOfCameras(numberOfCameras);

    // Open first available uEye camera. You can also open a camera over its serial number by passing a string as argument      
    ret = m_uEyeCamera->InitCamera();

    // Load parameters from user memory. uEye only supports one memory in the camera  
    //ret = m_uEyeCamera->LoadParametersFromCameraMemory();
    
    // Pixel format
    int colorMode = 0;
    int bitsPerPixel = 0;
    ret = m_uEyeCamera->SetColorMode(IS_CM_BGRA8_PACKED);
    //ret = m_uEyeCamera->SetColorMode(IS_CM_SENSOR_RAW8);
    ret = m_uEyeCamera->GetColorMode(colorMode);
    ret = m_uEyeCamera->GetBitsPerPixel(colorMode, bitsPerPixel);

    // Get AOI boundaries
    int x = 0;
    int y = 0;
    int width = 0;
    int height = 0;
    ret = m_uEyeCamera->GetImageAoiMin(x, y, width, height);

    // The max x and y pos depends on the current setting of the width and height
    // The max width and height depends on the current setting of the x and y position
    ret = m_uEyeCamera->GetImageAoiMax(x, y, width, height);
    ret = m_uEyeCamera->GetImageAoi(x, y, width, height);

    // Set new AOI
    //x = 64;
    //y = 64;
    //width = 512;
    //height = 512;
    //ret = m_uEyeCamera->SetImageAoi(x, y, width, height);

    // Get info about the sensor and the interface
    CAMERA_INFORMATION cameraInfo;
    ret = m_uEyeCamera->GetCameraInfo(cameraInfo);
        
    // Alloc memory and create a sequence
    ret = m_uEyeCamera->AllocImageBuffersAndCreateSequence(width, height, bitsPerPixel);

    // The camera image will be scaled to fit to the window
    ret = m_uEyeCamera->SetRenderType(DISPLAY_FIT_TO_WINDOW);
    //ret = m_uEyeCamera->SetRenderType(DISPLAY_NORMAL);

    // Init frame event
    ret = m_uEyeCamera->InitEvents();

    UpdateSliderFramerate_uEye();
    UpdateSliderExposure_uEye();
    
    // Get trigger delay values
    int triggerDelay_us = 0;
    ret = m_uEyeCamera->GetTriggerDelay_us(triggerDelay_us);
    ret = m_uEyeCamera->GetTriggerDelayMin_us(triggerDelay_us);
    ret = m_uEyeCamera->GetTriggerDelayMax_us(triggerDelay_us);
    ret = m_uEyeCamera->GetTriggerDelayInc_us(triggerDelay_us);

    // Set trigger delay to 1 sec
    //ret = m_uEyeCamera->SetTriggerDelay_us(1000000);

    // Disable trigger delay
    //ret = m_uEyeCamera->SetTriggerDelay_us(0);

    // Master gain
    double gain = 0;
    ret = m_uEyeCamera->GetGainMin(GAIN_MASTER, gain);
    ret = m_uEyeCamera->GetGainMax(GAIN_MASTER, gain);
    //ret = m_uEyeCamera->SetGain(GAIN_MASTER, 4.0);
    ret = m_uEyeCamera->GetGain(GAIN_MASTER, gain);

    // Red gain
    ret = m_uEyeCamera->GetGainMin(GAIN_RED, gain);
    ret = m_uEyeCamera->GetGainMax(GAIN_RED, gain);
    //ret = m_uEyeCamera->SetGain(GAIN_RED, 8.0);
    ret = m_uEyeCamera->GetGain(GAIN_RED, gain);

    // Green gain
    ret = m_uEyeCamera->GetGainMin(GAIN_GREEN, gain);
    ret = m_uEyeCamera->GetGainMax(GAIN_GREEN, gain);
    //ret = m_uEyeCamera->SetGain(GAIN_GREEN, 8.0);
    ret = m_uEyeCamera->GetGain(GAIN_GREEN, gain);

    // Blue gain
    ret = m_uEyeCamera->GetGainMin(GAIN_BLUE, gain);
    ret = m_uEyeCamera->GetGainMax(GAIN_BLUE, gain);
    //ret = m_uEyeCamera->SetGain(GAIN_BLUE, 8.0);
    ret = m_uEyeCamera->GetGain(GAIN_BLUE, gain);


    // AES/AGC

    int autoExposure = 1;
    //ret = m_uEyeCamera->SetAutoExposure(autoExposure);
    ret = m_uEyeCamera->GetAutoExposure(autoExposure);

    int autoGain = 1;
    //ret = m_uEyeCamera->SetAutoGain(autoGain);
    ret = m_uEyeCamera->GetAutoGain(autoGain);

    // Auto brightness once
    int once = 1;
    //ret = m_uEyeCamera->SetAutoBrightnessOnce(once);
    ret = m_uEyeCamera->GetAutoBrightnessOnce(once);

    // Grey world algorithm with a mean value
    int reference = 127;
    //ret = m_uEyeCamera->SetAutoBrightnessReference(reference);
    ret = m_uEyeCamera->GetAutoBrightnessReference(reference);

    // Speed of the controller from 0 to 100
    int speed = 100;
    //ret = m_uEyeCamera->SetAutoBrightnessSpeed(speed);
    ret = m_uEyeCamera->GetAutoBrightnessSpeed(speed);

    // Do not use all the frames and skip some during the control
    int skipframes = 0;
    ret = m_uEyeCamera->GetAutoBrightnessSkipframesMin(skipframes);
    ret = m_uEyeCamera->GetAutoBrightnessSkipframesMax(skipframes);
    //ret = m_uEyeCamera->SetAutoBrightnessSkipframes(4);
    ret = m_uEyeCamera->GetAutoBrightnessSkipframes(skipframes);
    
    int hysteresis = 0;
    ret = m_uEyeCamera->GetAutoBrightnessHysteresisMin(hysteresis);
    ret = m_uEyeCamera->GetAutoBrightnessHysteresisMax(hysteresis);
    //ret = m_uEyeCamera->SetAutoBrightnessHysteresis(2);
    ret = m_uEyeCamera->GetAutoBrightnessHysteresis(hysteresis);
    
    // Save all current parameters in the internal camera memory
    //ret = m_uEyeCamera->SaveParametersInCameraMemory();

    // Subsampling
    int subX = 2;
    int subY = 2;
    //ret = m_uEyeCamera->SetSubsamplingFactor(subX, subY);
    ret = m_uEyeCamera->GetSubsamplingFactor(subX, subY);

    // Start image acquisition
    OnButtonStart_uEye();


    /********************************************************************************************************/
    /*                                                                                                      */
    /*       peak SDK                                                                                       */
    /*                                                                                                      */
    /********************************************************************************************************/
    
   	// Open peak camera
    m_peakCamera = std::make_shared<IdsSherpa>(GetDlgItem(IDC_DISPLAY_PEAK)->m_hWnd);
    //m_peakCamera = std::make_shared<IdsSherpa>();
    //m_peakCamera->SetWindowHandle(GetDlgItem(IDC_DISPLAY_PEAK)->m_hWnd);

    // Get number of connected vision cameras
    ret = m_peakCamera->GetNumberOfCameras(numberOfCameras);

    // Open first available vision camera. You can also open a camera over its serial number by passing a string as argument
    ret = m_peakCamera->InitCamera();

    // Load parameters from user memory 0
    //ret = m_peakCamera->LoadParametersFromCameraMemory(USER_SET_0);

    // Pixel format
    peak::ipl::PixelFormatName pixelFormat;
    //ret = m_peakCamera->SetColorModeSource(peak::ipl::PixelFormatName::BayerRG8);
    //ret = m_peakCamera->SetColorModeDest(peak::ipl::PixelFormatName::BGRa8);
    //ret = m_peakCamera->GetColorModeSource(pixelFormat);
    //ret = m_peakCamera->GetColorModeDest(pixelFormat);

    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::BayerRG8);
    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::Mono8);
    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::BGR8);
    ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::BGRa8);
    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::RGB8);
    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::RGBa8);
    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::BayerRG10);
    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::Mono10);
    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::BGR10); // cannot be displayed without reducing bits
    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::BGRa10); // cannot be displayed without reducing bits
    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::RGB10); // cannot be displayed without reducing bits
    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::RGBa10); // cannot be displayed without reducing bits
    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::RGB10p32);
    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::BayerRG12);
    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::Mono12);
    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::BGR12); // cannot be displayed without reducing bits
    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::BGRa12); // cannot be displayed without reducing bits
    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::RGB12); // cannot be displayed without reducing bits
    //ret = m_peakCamera->SetColorMode(peak::ipl::PixelFormatName::RGBa12); // cannot be displayed without reducing bits

    ret = m_peakCamera->GetColorMode(pixelFormat);
    ret = m_peakCamera->GetBitsPerPixel(pixelFormat, bitsPerPixel);
    
    // Get AOI boundaries
    ret = m_peakCamera->GetImageAoiMin(x, y, width, height);
    ret = m_peakCamera->GetImageAoiMax(x, y, width, height);
    ret = m_peakCamera->GetImageAoi(x, y, width, height);

    // Set new AOI
    //x = 64;
    //y = 64;
    //width = 512;
    //height = 512;
    //ret = m_peakCamera->SetImageAoi(x, y, width, height);

    // Get info about the sensor and the interface
    ret = m_peakCamera->GetCameraInfo(cameraInfo);
    
    // Alloc memory and put it into a queue with 10 buffers
    ret = m_peakCamera->AllocImageBuffersAndCreateSequence(10);

    // The camera image will be scaled to fit to the window
    ret = m_peakCamera->SetRenderType(DISPLAY_FIT_TO_WINDOW);
    //ret = m_peakCamera->SetRenderType(DISPLAY_NORMAL);
    
    UpdateSliderFramerate_peak();
    UpdateSliderExposure_peak();

    // Get trigger delay values
    ret = m_peakCamera->GetTriggerDelay_us(triggerDelay_us);
    ret = m_peakCamera->GetTriggerDelayMin_us(triggerDelay_us);
    ret = m_peakCamera->GetTriggerDelayMax_us(triggerDelay_us);
    ret = m_peakCamera->GetTriggerDelayInc_us(triggerDelay_us);

    // Set trigger delay to 1 sec
    //ret = m_peakCamera->SetTriggerDelay_us(1000000);

    // Disable trigger delay
    //ret = m_peakCamera->SetTriggerDelay_us(0);

    // Analog master gain
    ret = m_peakCamera->GetGainMin(GAIN_MASTER_ANALOG, gain);
    ret = m_peakCamera->GetGainMax(GAIN_MASTER_ANALOG, gain);
    //ret = m_peakCamera->SetGain(GAIN_MASTER_ANALOG, 5.0);
    ret = m_peakCamera->GetGain(GAIN_MASTER_ANALOG, gain);

    // Digital master gain
    ret = m_peakCamera->GetGainMin(GAIN_MASTER_DIGITAL, gain);
    ret = m_peakCamera->GetGainMax(GAIN_MASTER_DIGITAL, gain);
    //ret = m_peakCamera->SetGain(GAIN_MASTER_DIGITAL, 8.0);
    ret = m_peakCamera->GetGain(GAIN_MASTER_DIGITAL, gain);

    // Red gain
    ret = m_peakCamera->GetGainMin(GAIN_RED, gain);
    ret = m_peakCamera->GetGainMax(GAIN_RED, gain);
    //ret = m_peakCamera->SetGain(GAIN_RED, 8.0);
    ret = m_peakCamera->GetGain(GAIN_RED, gain);

    // Green gain
    ret = m_peakCamera->GetGainMin(GAIN_GREEN, gain);
    ret = m_peakCamera->GetGainMax(GAIN_GREEN, gain);
    //ret = m_peakCamera->SetGain(GAIN_GREEN, 8.0);
    ret = m_peakCamera->GetGain(GAIN_GREEN, gain);

    // Blue gain
    ret = m_peakCamera->GetGainMin(GAIN_BLUE, gain);
    ret = m_peakCamera->GetGainMax(GAIN_BLUE, gain);
    //ret = m_peakCamera->SetGain(GAIN_BLUE, 8.0);
    ret = m_peakCamera->GetGain(GAIN_BLUE, gain);

    // AES/AGC

    autoExposure = AUTO_EXPOSURE_CONTINUOUS;
    //autoExposure = AUTO_EXPOSURE_ONCE;
    //ret = m_peakCamera->SetAutoExposure(autoExposure);
    ret = m_peakCamera->GetAutoExposure(autoExposure);

    autoGain = AUTO_GAIN_CONTINUOUS;
    //autoGain = AUTO_GAIN_ONCE;
    //ret = m_peakCamera->SetAutoGain(autoGain);
    ret = m_peakCamera->GetAutoGain(autoGain);

    // See description of the parameters in the feature reference

    double autoPercentile;
    ret = m_peakCamera->GetBrightnessAutoPercentileMin(autoPercentile);
    ret = m_peakCamera->GetBrightnessAutoPercentileMax(autoPercentile);
    //ret = m_peakCamera->SetBrightnessAutoPercentile(autoPercentile / 2);
    ret = m_peakCamera->GetBrightnessAutoPercentile(autoPercentile);

    int autoTarget;
    ret = m_peakCamera->GetBrightnessAutoTargetMin(autoTarget);
    ret = m_peakCamera->GetBrightnessAutoTargetMax(autoTarget);
    //ret = m_peakCamera->SetBrightnessAutoTarget(autoTarget / 2);
    ret = m_peakCamera->GetBrightnessAutoTarget(autoTarget);

    int autoTargetTolerance;
    ret = m_peakCamera->GetBrightnessAutoTargetToleranceMin(autoTargetTolerance);
    ret = m_peakCamera->GetBrightnessAutoTargetToleranceMax(autoTargetTolerance);
    //ret = m_peakCamera->SetBrightnessAutoTargetTolerance(autoTargetTolerance / 2);
    ret = m_peakCamera->GetBrightnessAutoTargetTolerance(autoTargetTolerance);

    // Save current parameters in the user memory 1
    //ret = m_peakCamera->SaveParametersInCameraMemory(USER_SET_1);

    // Subsampling
    //ret = m_peakCamera->SetSubsamplingFactor(2, 2);
    ret = m_peakCamera->GetSubsamplingFactor(subX, subY);

    // Start image acquisition 
    OnButtonStart_peak();

	return true;
}


void IdsSherpa_uEye_dlg::UpdateSliderFramerate_uEye()
{
    CString str;

    double min = 0;
    INT ret = m_uEyeCamera->GetFramerateMin(min);
    str.Format(L"%.1f [fps]", min);
    m_labelFramerateMin_uEye.SetWindowText(str);

    double max = 0;
    ret = m_uEyeCamera->GetFramerateMax(max);
    str.Format(L"%.1f [fps]", max);
    m_labelFramerateMax_uEye.SetWindowText(str);

    double framerate = 0;
    ret = m_uEyeCamera->GetFramerate(framerate);
    str.Format(L"%.1f [fps]", framerate);
    m_labelFramerate_uEye.SetWindowText(str);

    int numberOfSteps = static_cast<int>((max - min) / 0.1);

    m_sliderFramerate_uEye.SetRange(0, numberOfSteps, 1);

    // Current position
    // framerate = min + (pos * 0.1) ==> pos = (framerate - min) / 0.1
    int position = static_cast<int>((framerate - min) / 0.1);

    m_sliderFramerate_uEye.SetPos(position);
}


void IdsSherpa_uEye_dlg::UpdateSliderExposure_uEye()
{
    CString str;

    double min = 0;
    INT ret = m_uEyeCamera->GetExposureTimeMin_ms(min);
    str.Format(L"%.2f [ms]", min);
    m_labelExposureMin_uEye.SetWindowText(str);

    double max = 0;
    ret = m_uEyeCamera->GetExposureTimeMax_ms(max);
    str.Format(L"%.2f [ms]", max);
    m_labelExposureMax_uEye.SetWindowText(str);

    double exposure = 0;
    ret = m_uEyeCamera->GetExposureTime_ms(exposure);
    str.Format(L"%.2f [ms]", exposure);
    m_labelExposure_uEye.SetWindowText(str);

    int numberOfSteps = static_cast<int>((max - min) / 0.01);

    m_sliderExposure_uEye.SetRange(0, numberOfSteps, 1);

    // Current position
    // exposure = min + (pos * 0.01) ==> pos = (exposure - min) / 0.01
    int position = static_cast<int>((exposure - min) / 0.01);

    m_sliderExposure_uEye.SetPos(position);
}


void IdsSherpa_uEye_dlg::UpdateComboTriggerMode_uEye()
{
    m_comboTriggerMode_uEye.ResetContent();

    m_comboTriggerMode_uEye.InsertString(0, L"Off");
    m_comboTriggerMode_uEye.SetItemData(0, IS_SET_TRIGGER_OFF);

    m_comboTriggerMode_uEye.InsertString(1, L"Trigger hardware falling");
    m_comboTriggerMode_uEye.SetItemData(1, IS_SET_TRIGGER_HI_LO);

    m_comboTriggerMode_uEye.InsertString(2, L"Trigger hardware rising");
    m_comboTriggerMode_uEye.SetItemData(2, IS_SET_TRIGGER_LO_HI);

    // The software trigger in uEye works different than the software trigger in peak:
    // In uEye it is an continuous mechanism that triggers images via software as fast as possible
    // There is also an manual "force trigger" command that only works in combination with a hardware trigger 

    m_comboTriggerMode_uEye.InsertString(3, L"Trigger software");
    m_comboTriggerMode_uEye.SetItemData(3, IS_SET_TRIGGER_SOFTWARE);

    int triggerMode = 0;
    bool ret = m_uEyeCamera->GetTriggerMode(triggerMode);

    for (int i = 0; i < m_comboTriggerMode_uEye.GetCount(); i++)
    {
        if (triggerMode == m_comboTriggerMode_uEye.GetItemData(i))
        {
            m_comboTriggerMode_uEye.SetCurSel(i);
            break;
        }
    }

    m_comboTriggerMode_uEye.EnableWindow(!m_uEyeCamera->IsCapturing());
}


void IdsSherpa_uEye_dlg::UpdateSliderFramerate_peak()
{
    CString str;

    double min = 0;
    INT ret = m_peakCamera->GetFramerateMin(min);
    str.Format(L"%.1f [fps]", min);
    m_labelFramerateMin_peak.SetWindowText(str);

    double max = 0;
    ret = m_peakCamera->GetFramerateMax(max);
    str.Format(L"%.1f [fps]", max);
    m_labelFramerateMax_peak.SetWindowText(str);

    double framerate = 0;
    ret = m_peakCamera->GetFramerate(framerate);
    str.Format(L"%.1f [fps]", framerate);
    m_labelFramerate_peak.SetWindowText(str);

    int numberOfSteps = static_cast<int>((max - min) / 0.1);

    m_sliderFramerate_peak.SetRange(0, numberOfSteps, 1);

    // Current position
    // framerate = min + (pos * 0.1) ==> pos = (framerate - min) / 0.1
    int position = static_cast<int>((framerate - min) / 0.1);

    m_sliderFramerate_peak.SetPos(position);
}


void IdsSherpa_uEye_dlg::UpdateSliderExposure_peak()
{
    CString str;

    double min = 0;
    INT ret = m_peakCamera->GetExposureTimeMin_ms(min);
    str.Format(L"%.2f [ms]", min);
    m_labelExposureMin_peak.SetWindowText(str);

    double max = 0;
    ret = m_peakCamera->GetExposureTimeMax_ms(max);
    str.Format(L"%.2f [ms]", max);
    m_labelExposureMax_peak.SetWindowText(str);

    double exposure = 0;
    ret = m_peakCamera->GetExposureTime_ms(exposure);
    str.Format(L"%.2f [ms]", exposure);
    m_labelExposure_peak.SetWindowText(str);

    int numberOfSteps = static_cast<int>((max - min) / 0.01);

    m_sliderExposure_peak.SetRange(0, numberOfSteps, 1);

    // Current position
    // exposure = min + (pos * 0.01) ==> pos = (exposure - min) / 0.01
    int position = static_cast<int>((exposure - min) / 0.01);

    m_sliderExposure_peak.SetPos(position);
}


void IdsSherpa_uEye_dlg::UpdateComboTriggerMode_peak()
{
    m_comboTriggerMode_peak.ResetContent();

    m_comboTriggerMode_peak.InsertString(0, L"Off");
    m_comboTriggerMode_peak.SetItemData(0, TRIGGER_OFF);

    m_comboTriggerMode_peak.InsertString(1, L"Trigger hardware falling");
    m_comboTriggerMode_peak.SetItemData(1, TRIGGER_HI_LO);

    m_comboTriggerMode_peak.InsertString(2, L"Trigger hardware rising");
    m_comboTriggerMode_peak.SetItemData(2, TRIGGER_LO_HI);

    m_comboTriggerMode_peak.InsertString(3, L"Trigger software");
    m_comboTriggerMode_peak.SetItemData(3, TRIGGER_SOFTWARE);

    int triggerMode = 0;
    bool ret = m_peakCamera->GetTriggerMode(triggerMode);

    for (int i = 0; i < m_comboTriggerMode_peak.GetCount(); i++)
    {
        if (triggerMode == m_comboTriggerMode_peak.GetItemData(i))
        {
            m_comboTriggerMode_peak.SetCurSel(i);
            break;
        }
    }

    m_comboTriggerMode_peak.EnableWindow(!m_peakCamera->IsCapturing());
}


void IdsSherpa_uEye_dlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
    if ((CSliderCtrl*)pScrollBar == &m_sliderFramerate_uEye)
    {
        int value = m_sliderFramerate_uEye.GetPos();

        double min = 0;
        INT ret = m_uEyeCamera->GetFramerateMin(min);

        double framerate = min + (value * 0.1);
        CString str;
        str.Format(L"%.1f [fps]", framerate);
        m_labelFramerate_uEye.SetWindowText(str);

        ret = m_uEyeCamera->SetFramerate(framerate);

        UpdateSliderExposure_uEye();
    }
    else if ((CSliderCtrl*)pScrollBar == &m_sliderExposure_uEye)
    {
        int value = m_sliderExposure_uEye.GetPos();

        double min = 0;
        INT ret = m_uEyeCamera->GetExposureTimeMin_ms(min);

        double exposure = min + (value * 0.01);
        CString str;
        str.Format(L"%.2f [ms]", exposure);
        m_labelExposure_uEye.SetWindowText(str);

        ret = m_uEyeCamera->SetExposureTime_ms(exposure);
    }
    else if ((CSliderCtrl*)pScrollBar == &m_sliderFramerate_peak)
    {
        int value = m_sliderFramerate_peak.GetPos();

        double min = 0;
        INT ret = m_peakCamera->GetFramerateMin(min);

        double framerate = min + (value * 0.1);
        CString str;
        str.Format(L"%.1f [fps]", framerate);
        m_labelFramerate_peak.SetWindowText(str);

        ret = m_peakCamera->SetFramerate(framerate);
    }
    else if ((CSliderCtrl*)pScrollBar == &m_sliderExposure_peak)
    {
        int value = m_sliderExposure_peak.GetPos();

        double min = 0;
        INT ret = m_peakCamera->GetExposureTimeMin_ms(min);

        double exposure = min + (value * 0.01);
        CString str;
        str.Format(L"%.2f [ms]", exposure);
        m_labelExposure_peak.SetWindowText(str);

        ret = m_peakCamera->SetExposureTime_ms(exposure);

        UpdateSliderFramerate_peak();
    }
        
    CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}


void IdsSherpa_uEye_dlg::OnComboTriggerMode_uEye()
{
    int triggerMode = static_cast<int>(m_comboTriggerMode_uEye.GetItemData(m_comboTriggerMode_uEye.GetCurSel()));
    if (!m_uEyeCamera->SetTriggerMode(triggerMode))
    {
        UpdateComboTriggerMode_uEye();
        return;
    }

    // Set the trigger timeout to 1000 ms to avoid blocked GUI
    if (!m_uEyeCamera->SetTriggerTimeout_ms(1000))
    {
        UpdateComboTriggerMode_uEye();
        return;
    }

    UpdateComboTriggerMode_uEye();
}


void IdsSherpa_uEye_dlg::OnComboTriggerMode_peak()
{
    int triggerMode = static_cast<int>(m_comboTriggerMode_peak.GetItemData(m_comboTriggerMode_peak.GetCurSel()));
    if (!m_peakCamera->SetTriggerMode(triggerMode))
    {
        UpdateComboTriggerMode_peak();
        return;
    }

    if (!m_peakCamera->SetTriggerTimeout_ms(1000))
    {
        UpdateComboTriggerMode_peak();
        return;
    }

    UpdateComboTriggerMode_peak();
}


void IdsSherpa_uEye_dlg::OnBnClickedButtonForceTriggerUeye()
{
    m_uEyeCamera->ForceTrigger();
}


void IdsSherpa_uEye_dlg::OnBnClickedButtonForceTriggerPeak()
{
    m_peakCamera->ForceTrigger();
}


void IdsSherpa_uEye_dlg::OnSysCommand(UINT nID, LPARAM lParam)
{
    CDialog::OnSysCommand(nID, lParam);
}


void IdsSherpa_uEye_dlg::OnPaint()
{
    if (IsIconic())
    {
        CPaintDC dc(this); // device context for painting

        SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

        // Center icon in client rectangle
        int cxIcon = GetSystemMetrics(SM_CXICON);
        int cyIcon = GetSystemMetrics(SM_CYICON);
        CRect rect;
        GetClientRect(&rect);
        int x = (rect.Width() - cxIcon + 1) / 2;
        int y = (rect.Height() - cyIcon + 1) / 2;

        // Draw the icon
        dc.DrawIcon(x, y, m_hIcon);
    }
    else
    {
        CDialog::OnPaint();
    }
}


void IdsSherpa_uEye_dlg::CloseAll()
{
    if (m_peakCamera)
    {
        INT ret = m_peakCamera->StopCapture();
        ret = m_peakCamera->ExitEvents();
        ret = m_peakCamera->FreeImageBuffersAndClearSequence();
        ret = m_peakCamera->ExitCamera();

        m_peakCamera = nullptr;
    }

    if (m_uEyeCamera)
    {
        INT ret = m_uEyeCamera->StopCapture();   
        ret = m_uEyeCamera->ExitEvents();
        ret = m_uEyeCamera->FreeImageBuffersAndClearSequence();
        ret = m_uEyeCamera->ExitCamera();

        m_uEyeCamera = nullptr;
    }
}


void IdsSherpa_uEye_dlg::OnClose()
{
    CloseAll();
    CDialog::OnClose();
}


HCURSOR IdsSherpa_uEye_dlg::OnQueryDragIcon()
{
    return (HCURSOR) m_hIcon;
}


void IdsSherpa_uEye_dlg::OnButtonStart_uEye()
{
    m_uEyeCamera->StartCapture();
    UpdateComboTriggerMode_uEye();

    GetDlgItem(IDC_BUTTON_START_UEYE)->EnableWindow(!m_uEyeCamera->IsCapturing());
    GetDlgItem(IDC_BUTTON_STOP_UEYE)->EnableWindow(m_uEyeCamera->IsCapturing());

    int triggerMode = static_cast<int>(m_comboTriggerMode_uEye.GetItemData(m_comboTriggerMode_uEye.GetCurSel()));
    if ((IS_SET_TRIGGER_HI_LO == triggerMode) && m_uEyeCamera->IsCapturing())
    {
        GetDlgItem(IDC_BUTTON_FORCE_TRIGGER_UEYE)->EnableWindow(TRUE);
    }
    else if ((IS_SET_TRIGGER_LO_HI == triggerMode) && m_uEyeCamera->IsCapturing())
    {
        GetDlgItem(IDC_BUTTON_FORCE_TRIGGER_UEYE)->EnableWindow(TRUE);
    }
    else
    {
        GetDlgItem(IDC_BUTTON_FORCE_TRIGGER_UEYE)->EnableWindow(FALSE);
    }
}


void IdsSherpa_uEye_dlg::OnButtonStop_uEye()
{
    m_uEyeCamera->StopCapture();
    UpdateComboTriggerMode_uEye();

    GetDlgItem(IDC_BUTTON_START_UEYE)->EnableWindow(!m_uEyeCamera->IsCapturing());
    GetDlgItem(IDC_BUTTON_STOP_UEYE)->EnableWindow(m_uEyeCamera->IsCapturing());

    int triggerMode = static_cast<int>(m_comboTriggerMode_uEye.GetItemData(m_comboTriggerMode_uEye.GetCurSel()));
    if ((IS_SET_TRIGGER_HI_LO == triggerMode) && m_uEyeCamera->IsCapturing())
    {
        GetDlgItem(IDC_BUTTON_FORCE_TRIGGER_UEYE)->EnableWindow(TRUE);
    }
    else if ((IS_SET_TRIGGER_LO_HI == triggerMode) && m_uEyeCamera->IsCapturing())
    {
        GetDlgItem(IDC_BUTTON_FORCE_TRIGGER_UEYE)->EnableWindow(TRUE);
    }
    else
    {
        GetDlgItem(IDC_BUTTON_FORCE_TRIGGER_UEYE)->EnableWindow(FALSE);
    }
}


void IdsSherpa_uEye_dlg::OnButtonStart_peak()
{
    m_peakCamera->StartCapture();
    UpdateComboTriggerMode_peak();

    GetDlgItem(IDC_BUTTON_START_PEAK)->EnableWindow(!m_peakCamera->IsCapturing());
    GetDlgItem(IDC_BUTTON_STOP_PEAK)->EnableWindow(m_peakCamera->IsCapturing());

    int triggerMode = static_cast<int>(m_comboTriggerMode_peak.GetItemData(m_comboTriggerMode_peak.GetCurSel()));
    if ((TRIGGER_SOFTWARE == triggerMode) && m_peakCamera->IsCapturing())
    {
        GetDlgItem(IDC_BUTTON_FORCE_TRIGGER_PEAK)->EnableWindow(TRUE);
    }
    else
    {
        GetDlgItem(IDC_BUTTON_FORCE_TRIGGER_PEAK)->EnableWindow(FALSE);
    }
}


void IdsSherpa_uEye_dlg::OnButtonStop_peak()
{
    m_peakCamera->StopCapture();
    UpdateComboTriggerMode_peak();

    GetDlgItem(IDC_BUTTON_START_PEAK)->EnableWindow(!m_peakCamera->IsCapturing());
    GetDlgItem(IDC_BUTTON_STOP_PEAK)->EnableWindow(m_peakCamera->IsCapturing());

    int triggerMode = static_cast<int>(m_comboTriggerMode_peak.GetItemData(m_comboTriggerMode_peak.GetCurSel()));
    if ((TRIGGER_SOFTWARE == triggerMode) && m_peakCamera->IsCapturing())
    {
        GetDlgItem(IDC_BUTTON_FORCE_TRIGGER_PEAK)->EnableWindow(TRUE);
    }
    else
    {
        GetDlgItem(IDC_BUTTON_FORCE_TRIGGER_PEAK)->EnableWindow(FALSE);
    }
}


void IdsSherpa_uEye_dlg::OnButtonExit()
{
    CloseAll();
    PostQuitMessage(0);
}
