//===========================================================================//
//                                                                           //
//  Copyright (C) 2004 - 2019                                                //
//  IDS Imaging Development Systems GmbH                                     //
//  Dimbacherstr. 6-8                                                        //
//  D-74182 Obersulm-Willsbach                                               //
//                                                                           //
//  The information in this document is subject to change without            //
//  notice and should not be construed as a commitment by IDS Imaging GmbH.  //
//  IDS Imaging GmbH does not assume any responsibility for any errors       //
//  that may appear in this document.                                        //
//                                                                           //
//  This document, or source code, is provided solely as an example          //
//  of how to utilize IDS software libraries in a sample application.        //
//  IDS Imaging GmbH does not assume any responsibility for the use or       //
//  reliability of any portion of this document or the described software.   //
//                                                                           //
//  General permission to copy or modify, but not for profit, is hereby      //
//  granted,  provided that the above copyright notice is included and       //
//  reference made to the fact that reproduction privileges were granted     //
//  by IDS Imaging GmbH.                                                     //
//                                                                           //
//  IDS cannot assume any responsibility for the use or misuse of any        //
//  portion of this software for other than its intended diagnostic purpose  //
//  in calibrating and testing IDS manufactured cameras and software.        //
//                                                                           //
//===========================================================================//

#include "stdafx.h"

#include "uEye.h"
#include "IdsuEye.h"

#include <iostream>


IdsuEye::IdsuEye(HWND hWnd)
{
    m_hCam = NULL;
    m_hWnd = hWnd;
    m_threadRunning = FALSE;
    m_renderType = IS_RENDER_FIT_TO_WINDOW;
}


IdsuEye::~IdsuEye()
{
    StopCapture();
}


bool IdsuEye::InitCamera(std::string serialNumber)
{
    UEYE_CAMERA_LIST* uEyeCameraList = new UEYE_CAMERA_LIST;
    if (NULL == uEyeCameraList)
    {
        return false;
    }

    // Initialize dwCount with zero. After the function call
    // dwCount will hold the number of cameras in the list
    uEyeCameraList->dwCount = 0;
    if (IS_SUCCESS != is_GetCameraList(uEyeCameraList))
    {
        delete[] uEyeCameraList;
        return false;
    }

    // Save number and delete list
    DWORD numCameras = uEyeCameraList->dwCount;
    delete uEyeCameraList;

    uEyeCameraList = (UEYE_CAMERA_LIST*)new char[sizeof(DWORD) + numCameras * sizeof(UEYE_CAMERA_INFO)];
    uEyeCameraList->dwCount = numCameras;

    if (IS_SUCCESS != is_GetCameraList(uEyeCameraList))
    {
        delete[] uEyeCameraList;
        return false;
    }

    // If the serial number == "" -> we init the device ID with zero. in this case the first available camera is opened
    int deviceID = 0;

    if ("" != serialNumber)
    {
        deviceID = -1;
        for (int i = 0; i < (int)uEyeCameraList->dwCount; i++)
        {
            if (strcmp(uEyeCameraList->uci[i].SerNo, serialNumber.c_str()) == 0)
            {
                // Save corresponding device ID
                deviceID = uEyeCameraList->uci[i].dwDeviceID;
                break;
            }
        }
    }
    
    delete[] uEyeCameraList;

    if (-1 == deviceID)
    {
        return false;
    }

    // Open camera and save handle
    m_hCam = ((HIDS)(deviceID | IS_USE_DEVICE_ID));
    if (IS_SUCCESS != is_InitCamera(&m_hCam, m_hWnd))
    {
        return false;
    }

    return true;
}


bool IdsuEye::ExitCamera()
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (IS_SUCCESS != is_ExitCamera(m_hCam))
    {
        return false;
    }

    m_hCam = NULL;

    return true;
}


bool IdsuEye::GetNumberOfCameras(int& number)
{
    if (IS_SUCCESS != is_GetNumberOfCameras(&number))
    {
        return false;
    }

    return true;
}


bool IdsuEye::SetColorMode(int colorMode)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (IS_SUCCESS != is_SetColorMode(m_hCam, colorMode))
    {
        return false;
    }

    return true;
}


bool IdsuEye::GetColorMode(int& colorMode)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    colorMode = is_SetColorMode(m_hCam, IS_GET_COLOR_MODE);
    
    return true;
}


bool IdsuEye::GetBitsPerPixel(int colorMode, int& bitsPerPixel)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    switch (colorMode)
    {
        case IS_CM_SENSOR_RAW8:
        case IS_CM_MONO8:
        {
            bitsPerPixel = 8;
        }
        break;

        case IS_CM_BGR5_PACKED:
        case IS_CM_BGR565_PACKED:
        case IS_CM_UYVY_PACKED:
        case IS_CM_UYVY_MONO_PACKED:
        case IS_CM_UYVY_BAYER_PACKED:
        case IS_CM_CBYCRY_PACKED:
        case IS_CM_SENSOR_RAW10:
        case IS_CM_SENSOR_RAW12:
        case IS_CM_SENSOR_RAW16:
        case IS_CM_MONO10:
        case IS_CM_MONO12:
        case IS_CM_MONO16:
        {
            bitsPerPixel = 16;
        }
        break;

        case IS_CM_RGB8_PACKED:
        case IS_CM_BGR8_PACKED:
        case IS_CM_RGB8_PLANAR:
        {
            bitsPerPixel = 24;
        }
        break;

        case IS_CM_RGBA8_PACKED:
        case IS_CM_BGRA8_PACKED:
        case IS_CM_RGBY8_PACKED:
        case IS_CM_BGRY8_PACKED:
        case IS_CM_RGB10_PACKED:
        case IS_CM_BGR10_PACKED:
        {
            bitsPerPixel = 32;
        }
        break;

        case IS_CM_RGB10_UNPACKED:
        case IS_CM_BGR10_UNPACKED:
        case IS_CM_RGB12_UNPACKED:
        case IS_CM_BGR12_UNPACKED:
        {
            bitsPerPixel = 48;
        }
        break;
       
        case IS_CM_RGBA12_UNPACKED:
        case IS_CM_BGRA12_UNPACKED:
        {
            bitsPerPixel = 64;
        }
        break;

        default:
        {
            bitsPerPixel = 32;
        }
        break;
    }

    return true;
}


bool IdsuEye::GetImageAoiMin(int& x, int& y, int& width, int& height)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    IS_POINT_2D pos;

    if (IS_SUCCESS != is_AOI(m_hCam, IS_AOI_IMAGE_GET_POS_MIN, (void*)&pos, sizeof(pos)))
    {
        return false;
    }

    x = pos.s32X;
    y = pos.s32Y;


    IS_SIZE_2D size;

    if (IS_SUCCESS != is_AOI(m_hCam, IS_AOI_IMAGE_GET_SIZE_MIN, (void*)&size, sizeof(size)))
    {
        return false;
    }

    width = size.s32Width;
    height = size.s32Height;
    
    return true;
}


bool IdsuEye::GetImageAoiMax(int& x, int& y, int& width, int& height)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    IS_POINT_2D pos;

    if (IS_SUCCESS != is_AOI(m_hCam, IS_AOI_IMAGE_GET_POS_MAX, (void*)&pos, sizeof(pos)))
    {
        return false;
    }

    x = pos.s32X;
    y = pos.s32Y;


    IS_SIZE_2D size;

    if (IS_SUCCESS != is_AOI(m_hCam, IS_AOI_IMAGE_GET_SIZE_MAX, (void*)&size, sizeof(size)))
    {
        return false;
    }

    width = size.s32Width;
    height = size.s32Height;

    return true;
}


bool IdsuEye::GetImageAoi(int& x, int& y, int& width, int& height)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    IS_RECT aoi;

    if (IS_SUCCESS != is_AOI(m_hCam, IS_AOI_IMAGE_GET_AOI, (void*)&aoi, sizeof(aoi)))
    {
        return false;
    }

    x = aoi.s32X;
    y = aoi.s32Y;
    width = aoi.s32Width;
    height = aoi.s32Height;

    return true;
}


bool IdsuEye::SetImageAoi(int x, int y, int width, int height)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    int x_min = 0;
    int y_min = 0;
    int width_min = 0;
    int height_min = 0;

    if (!GetImageAoiMin(x_min, y_min, width_min, height_min))
    {
        return false;
    }

    IS_RECT aoi;
    aoi.s32X = x_min;
    aoi.s32Y = y_min;
    aoi.s32Width = width_min;
    aoi.s32Height = height_min;

    // Set the minimum AOI: After that all size restrictions allow setting any AOI possible
    if (IS_SUCCESS != is_AOI(m_hCam, IS_AOI_IMAGE_SET_AOI, (void*)&aoi, sizeof(aoi)))
    {
        return false;
    }

    int x_max = 0;
    int y_max = 0;
    int width_max = 0;
    int height_max = 0;

    if (!GetImageAoiMax(x_max, y_max, width_max, height_max))
    {
        return false;
    }

    // Some checks
    if (width < width_min)
    {
        width = width_min;
    }

    if (height < height_min)
    {
        height = height_min;
    }

    if ((x + width) > width_max)
    {
        width = width_max - x;
    }

    if (y + height > height_max)
    {
        height = height_max - y;
    }

    aoi.s32X = x;
    aoi.s32Y = y;
    aoi.s32Width = width;
    aoi.s32Height = height;

    if (IS_SUCCESS != is_AOI(m_hCam, IS_AOI_IMAGE_SET_AOI, (void*)&aoi, sizeof(aoi)))
    {
        return false;
    }

    return true;
}


bool IdsuEye::GetCameraInfo(CAMERA_INFORMATION& info)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    CAMINFO cameraInfo;
    if (IS_SUCCESS != is_GetCameraInfo(m_hCam, &cameraInfo))
    {
        return false;
    }

    SENSORINFO sensorInfo;
    if (IS_SUCCESS != is_GetSensorInfo(m_hCam, &sensorInfo))
    {
        return false;
    }

    info.serialNumber = cameraInfo.SerNo;
    info.manufacturer = cameraInfo.ID;

    std::map<int, std::string> mapCameraType;
    mapCameraType[IS_CAMERA_TYPE_UEYE_USB_SE] = "USB-SE";
    mapCameraType[IS_CAMERA_TYPE_UEYE_USB_ME] = "USB-ME";
    mapCameraType[IS_CAMERA_TYPE_UEYE_USB_LE] = "USB-LE";
    mapCameraType[IS_CAMERA_TYPE_UEYE_USB_ML] = "USB-ML";
    mapCameraType[IS_CAMERA_TYPE_UEYE_USB3_LE] = "USB3-LE";
    mapCameraType[IS_CAMERA_TYPE_UEYE_USB3_CP] = "USB3-CP";
    mapCameraType[IS_CAMERA_TYPE_UEYE_USB3_ML] = "USB3-ML";
    mapCameraType[IS_CAMERA_TYPE_UEYE_ETH_SE] = "ETH-SE";
    mapCameraType[IS_CAMERA_TYPE_UEYE_ETH_LE] = "ETH-LE";
    mapCameraType[IS_CAMERA_TYPE_UEYE_ETH_CP] = "ETH-CP";
    mapCameraType[IS_CAMERA_TYPE_UEYE_ETH_CP_R2] = "ETH-CP_R2";
    mapCameraType[IS_CAMERA_TYPE_UEYE_ETH_FA] = "ETH-FA";
    mapCameraType[IS_CAMERA_TYPE_UEYE_ETH_SE_R4] = "ETH-SE_R4";

    info.cameraType = mapCameraType[cameraInfo.Type];
    info.modelName = sensorInfo.strSensorName;
    info.maxSensorWidth = sensorInfo.nMaxWidth;
    info.maxSensorHeight = sensorInfo.nMaxHeight;
    info.bayerPattern = (sensorInfo.nColorMode == IS_COLORMODE_BAYER);

    if (BAYER_PIXEL_RED == sensorInfo.nUpperLeftBayerPixel)
    {
        info.upperLeftBayerPixel = "red";
    }
    else if (BAYER_PIXEL_GREEN == sensorInfo.nUpperLeftBayerPixel)
    {
        info.upperLeftBayerPixel = "green";
    }
    else if (BAYER_PIXEL_BLUE == sensorInfo.nUpperLeftBayerPixel)
    {
        info.upperLeftBayerPixel = "blue";
    }

    info.globalShutter = (TRUE == sensorInfo.bGlobShutter);

    return true;
}


bool IdsuEye::AllocImageBuffersAndCreateSequence(int width, int height, int bitsPerPixel)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    for (int i = 0; i < NUMBER_OF_SEQUENCE_BUFFERS; i++)
    {
        INT ret = is_AllocImageMem(m_hCam, width, height, bitsPerPixel, &(m_memory[i]), &(m_memoryId[i]));
        if (IS_SUCCESS != ret)
        {
            return false;
        }

        ret = is_AddToSequence(m_hCam, m_memory[i], m_memoryId[i]);
        if (IS_SUCCESS != ret)
        {
            return false;
        }
    }

    return true;
}


bool IdsuEye::FreeImageBuffersAndClearSequence()
{
    if (NULL == m_hCam)
    {
        return false;
    }

    for (int i = 0; i < NUMBER_OF_SEQUENCE_BUFFERS; i++)
    {
        INT ret = is_ClearSequence(m_hCam);
        if (IS_SUCCESS != ret)
        {
            return false;
        }

        ret = is_FreeImageMem(m_hCam, m_memory[i], m_memoryId[i]);
        if (IS_SUCCESS != ret)
        {
            return false;
        }
    }

    return true;
}


bool IdsuEye::SetRenderType(int renderType)
{
    if (DISPLAY_NORMAL == renderType)
    {
        m_renderType = IS_RENDER_NORMAL;
    }
    else if (DISPLAY_FIT_TO_WINDOW == renderType)
    {
        m_renderType = IS_RENDER_FIT_TO_WINDOW;
    }

    return true;
}


bool IdsuEye::InitEvents()
{
    if (NULL == m_hCam)
    {
        return false;
    }

    m_frameEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
    if (m_frameEvent == NULL)
    {
        return false;
    }

    INT ret = is_InitEvent(m_hCam, m_frameEvent, IS_SET_EVENT_FRAME);
    if (IS_SUCCESS != ret)
    {
        return false;
    }

    ret = is_EnableEvent(m_hCam, IS_SET_EVENT_FRAME);
    if (IS_SUCCESS != ret)
    {
        return false;
    }

    return true;
}


bool IdsuEye::ExitEvents()
{
    if (NULL == m_hCam)
    {
        return false;
    }

    INT ret = is_DisableEvent(m_hCam, IS_SET_EVENT_FRAME);
    if (IS_SUCCESS != ret)
    {
        return false;
    }

    ret = is_ExitEvent(m_hCam, IS_SET_EVENT_FRAME);
    if (IS_SUCCESS != ret)
    {
        return false;
    }

    if (TRUE != CloseHandle(m_frameEvent))
    {
        return false;
    }

    return true;
}


bool IdsuEye::StartCapture()
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (m_threadRunning)
    {
        return false;
    }    

    m_threadCapture = AfxBeginThread(CaptureWorker, this);

    if (IS_SUCCESS != is_CaptureVideo(m_hCam, IS_WAIT))
    {
        return false;
    }

    return true;
}


bool IdsuEye::StopCapture()
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (!m_threadRunning)
    {
        return false;
    }

    m_stopThread = TRUE;

    while (m_threadRunning)
    {
        Sleep(10);
    }

    if (IS_SUCCESS != is_StopLiveVideo(m_hCam, IS_FORCE_VIDEO_STOP))
    {
        return false;
    }
    
    return true;
}


bool IdsuEye::IsCapturing()
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (m_threadRunning)
    {
        return true;
    }
    else
    {
        return false;
    }
}


bool IdsuEye::GetFramerateMin(double& min_framerate)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double min;
    double max;
    double interval;
    if (IS_SUCCESS != is_GetFrameTimeRange(m_hCam, &min, &max, &interval))
    {
        return false;
    }

    // Framerate_min = 1 / frametime_max
    min_framerate = 1.0 / max;

    return true;
}


bool IdsuEye::GetFramerateMax(double& max_framerate)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double min;
    double max;
    double interval;
    if (IS_SUCCESS != is_GetFrameTimeRange(m_hCam, &min, &max, &interval))
    {
        return false;
    }

    // Framerate_max = 1 / frametime_min
    max_framerate = 1.0 / min;

    return true;
}


bool IdsuEye::GetFramerate(double& framerate)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (IS_SUCCESS != is_SetFrameRate(m_hCam, IS_GET_FRAMERATE, &framerate))
    {
        return false;
    }

    return true;
}


bool IdsuEye::SetFramerate(double framerate)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double framerate_set;
    if (IS_SUCCESS != is_SetFrameRate(m_hCam, framerate, &framerate_set))
    {
        return false;
    }

    return true;
}


bool IdsuEye::GetExposureTimeMin_ms(double& min)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (IS_SUCCESS != is_Exposure(m_hCam, IS_EXPOSURE_CMD_GET_EXPOSURE_RANGE_MIN, (void*)&min, sizeof(min)))
    {
        return false;
    }

    return true;
}


bool IdsuEye::GetExposureTimeMax_ms(double& max)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (IS_SUCCESS != is_Exposure(m_hCam, IS_EXPOSURE_CMD_GET_EXPOSURE_RANGE_MAX, (void*)&max, sizeof(max)))
    {
        return false;
    }

    return true;
}


bool IdsuEye::GetExposureTime_ms(double& exposure)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (IS_SUCCESS != is_Exposure(m_hCam, IS_EXPOSURE_CMD_GET_EXPOSURE, (void*)&exposure, sizeof(exposure)))
    {
        return false;
    }

    return true;
}


bool IdsuEye::SetExposureTime_ms(double exposure)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (IS_SUCCESS != is_Exposure(m_hCam, IS_EXPOSURE_CMD_SET_EXPOSURE, (void*)&exposure, sizeof(exposure)))
    {
        return false;
    }

    return true;
}


bool IdsuEye::GetTriggerMode(int& mode)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    mode = is_SetExternalTrigger(m_hCam, IS_GET_EXTERNALTRIGGER);

    return true;
}


bool IdsuEye::SetTriggerMode(int mode)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (IS_SUCCESS != is_SetExternalTrigger(m_hCam, mode))
    {
        return false;
    }

    return true;
}


bool IdsuEye::ForceTrigger()
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (IS_SUCCESS != is_ForceTrigger(m_hCam))
    {
        return false;
    }

    return true;
}


bool IdsuEye::SetTriggerTimeout_ms(int timeout)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    // The timeout value must be set in multiples of 10 ms
    if (IS_SUCCESS != is_SetTimeout(m_hCam, IS_TRIGGER_TIMEOUT, static_cast<int>(timeout / 10.0)))
    {
        return false;
    }

    return true;
}


bool IdsuEye::GetTriggerDelayMin_us(int& min)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    min = is_SetTriggerDelay(m_hCam, IS_GET_MIN_TRIGGER_DELAY);

    return true;
}


bool IdsuEye::GetTriggerDelayMax_us(int& max)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    max = is_SetTriggerDelay(m_hCam, IS_GET_MAX_TRIGGER_DELAY);

    return true;
}


bool IdsuEye::GetTriggerDelayInc_us(int& inc)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    inc = is_SetTriggerDelay(m_hCam, IS_GET_TRIGGER_DELAY_GRANULARITY);

    return true;
}


bool IdsuEye::GetTriggerDelay_us(int& delay)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    delay = is_SetTriggerDelay(m_hCam, IS_GET_TRIGGER_DELAY);
   
    return true;
}


bool IdsuEye::SetTriggerDelay_us(int delay)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (IS_SUCCESS != is_SetTriggerDelay(m_hCam, delay))
    {
        return false;
    }

    return true;
}


bool IdsuEye::GetGainMin(int type, double& min)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (GAIN_MASTER == type)
    {
        min = static_cast<double>(is_SetHWGainFactor(m_hCam, IS_INQUIRE_MASTER_GAIN_FACTOR, 0)) / 100.0;
    }
    else if (GAIN_RED == type)
    {
        min = static_cast<double>(is_SetHWGainFactor(m_hCam, IS_INQUIRE_RED_GAIN_FACTOR, 0)) / 100.0;
    }
    else if (GAIN_GREEN == type)
    {
        min = static_cast<double>(is_SetHWGainFactor(m_hCam, IS_INQUIRE_GREEN_GAIN_FACTOR, 0)) / 100.0;
    }
    else if (GAIN_BLUE == type)
    {
        min = static_cast<double>(is_SetHWGainFactor(m_hCam, IS_INQUIRE_BLUE_GAIN_FACTOR, 0)) / 100.0;
    }

    return true;
}


bool IdsuEye::GetGainMax(int type, double& max)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (GAIN_MASTER == type)
    {
        max = static_cast<double>(is_SetHWGainFactor(m_hCam, IS_INQUIRE_MASTER_GAIN_FACTOR, 100)) / 100.0;
    }
    else if (GAIN_RED == type)
    {
        max = static_cast<double>(is_SetHWGainFactor(m_hCam, IS_INQUIRE_RED_GAIN_FACTOR, 100)) / 100.0;
    }
    else if (GAIN_GREEN == type)
    {
        max = static_cast<double>(is_SetHWGainFactor(m_hCam, IS_INQUIRE_GREEN_GAIN_FACTOR, 100)) / 100.0;
    }
    else if (GAIN_BLUE == type)
    {
        max = static_cast<double>(is_SetHWGainFactor(m_hCam, IS_INQUIRE_BLUE_GAIN_FACTOR, 100)) / 100.0;
    }

    return true;
}


bool IdsuEye::GetGain(int type, double& gain)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (GAIN_MASTER == type)
    {
        gain = static_cast<double>(is_SetHWGainFactor(m_hCam, IS_GET_MASTER_GAIN_FACTOR, 0)) / 100.0;
    }
    if (GAIN_RED == type)
    {
        gain = static_cast<double>(is_SetHWGainFactor(m_hCam, IS_GET_RED_GAIN_FACTOR, 0)) / 100.0;
    }
    if (GAIN_GREEN == type)
    {
        gain = static_cast<double>(is_SetHWGainFactor(m_hCam, IS_GET_GREEN_GAIN_FACTOR, 0)) / 100.0;
    }
    if (GAIN_BLUE == type)
    {
        gain = static_cast<double>(is_SetHWGainFactor(m_hCam, IS_GET_BLUE_GAIN_FACTOR, 0)) / 100.0;
    }

    return true;
}


bool IdsuEye::SetGain(int type, double gain)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    INT reallySet = 0;

    if (GAIN_MASTER == type)
    {
        reallySet = is_SetHWGainFactor(m_hCam, IS_SET_MASTER_GAIN_FACTOR, static_cast<INT>((gain * 100.0)));
    }
    if (GAIN_RED == type)
    {
        reallySet = is_SetHWGainFactor(m_hCam, IS_SET_RED_GAIN_FACTOR, static_cast<INT>((gain * 100.0)));
    }
    if (GAIN_GREEN == type)
    {
        reallySet = is_SetHWGainFactor(m_hCam, IS_SET_GREEN_GAIN_FACTOR, static_cast<INT>((gain * 100.0)));
    }
    if (GAIN_BLUE == type)
    {
        reallySet = is_SetHWGainFactor(m_hCam, IS_SET_BLUE_GAIN_FACTOR, static_cast<INT>((gain * 100.0)));
    }

    if (-1 == reallySet)
    {
        return false;
    }

    return true;
}


bool IdsuEye::GetAutoExposure(int& enable)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp = 0;
    double dummy = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_GET_ENABLE_AUTO_SHUTTER, &temp, &dummy))
    {
        return false;
    }

    enable = static_cast<int>(temp);

    return true;
}


bool IdsuEye::SetAutoExposure(int enable)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp = static_cast<double>(enable);
    double dummy = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_SET_ENABLE_AUTO_SHUTTER, &temp, &dummy))
    {
        return false;
    }

    return true;
}


bool IdsuEye::GetAutoGain(int& enable)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp = 0;
    double dummy = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_GET_ENABLE_AUTO_GAIN, &temp, &dummy))
    {
        return false;
    }

    enable = static_cast<int>(temp);

    return true;
}


bool IdsuEye::SetAutoGain(int enable)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp = static_cast<double>(enable);
    double dummy = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_SET_ENABLE_AUTO_GAIN, &temp, &dummy))
    {
        return false;
    }

    return true;
}


bool IdsuEye::GetAutoBrightnessOnce(int& once)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp = 0;
    double dummy = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_GET_AUTO_BRIGHTNESS_ONCE, &temp, &dummy))
    {
        return false;
    }    

    once = static_cast<int>(temp);

    return true;
}


bool IdsuEye::SetAutoBrightnessOnce(int once)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp = static_cast<double>(once);
    double dummy = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_SET_AUTO_BRIGHTNESS_ONCE, &temp, &dummy))
    {
        return false;
    }

    return true;
}


bool IdsuEye::GetAutoBrightnessReference(int& value)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp = 0;
    double dummy = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_GET_AUTO_REFERENCE, &temp, &dummy))
    {
        return false;
    }

    value = static_cast<int>(temp);

    return true;
}


bool IdsuEye::SetAutoBrightnessReference(int value)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp = static_cast<double>(value);
    double dummy = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_SET_AUTO_REFERENCE, &temp, &dummy))
    {
        return false;
    }

    return true;
}


bool IdsuEye::GetAutoBrightnessSpeed(int& value)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp = 0;
    double dummy = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_GET_AUTO_SPEED, &temp, &dummy))
    {
        return false;
    }

    value = static_cast<int>(temp);

    return true;
}


bool IdsuEye::SetAutoBrightnessSpeed(int value)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp = static_cast<double>(value);
    double dummy = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_SET_AUTO_SPEED, &temp, &dummy))
    {
        return false;
    }

    return true;
}


bool IdsuEye::GetAutoBrightnessSkipframesMin(int& min)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp1 = 0;
    double temp2 = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_GET_AUTO_SKIPFRAMES_RANGE, &temp1, &temp2))
    {
        return false;
    }

    min = static_cast<int>(temp1);

    return true;
}


bool IdsuEye::GetAutoBrightnessSkipframesMax(int& max)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp1 = 0;
    double temp2 = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_GET_AUTO_SKIPFRAMES_RANGE, &temp1, &temp2))
    {
        return false;
    }

    max = static_cast<int>(temp2);

    return true;
}


bool IdsuEye::GetAutoBrightnessSkipframes(int& value)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp = 0;
    double dummy = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_GET_AUTO_SKIPFRAMES, &temp, &dummy))
    {
        return false;
    }

    value = static_cast<int>(temp);

    return true;
}


bool IdsuEye::SetAutoBrightnessSkipframes(int value)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp = static_cast<double>(value);
    double dummy = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_SET_AUTO_SKIPFRAMES, &temp, &dummy))
    {
        return false;
    }

    return true;
}


bool IdsuEye::GetAutoBrightnessHysteresisMin(int& min)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp1 = 0;
    double temp2 = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_GET_AUTO_HYSTERESIS_RANGE, &temp1, &temp2))
    {
        return false;
    }

    min = static_cast<int>(temp1);

    return true;
}


bool IdsuEye::GetAutoBrightnessHysteresisMax(int& max)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp1 = 0;
    double temp2 = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_GET_AUTO_HYSTERESIS_RANGE, &temp1, &temp2))
    {
        return false;
    }

    max = static_cast<int>(temp2);

    return true;
}


bool IdsuEye::GetAutoBrightnessHysteresis(int& value)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp = 0;
    double dummy = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_GET_AUTO_HYSTERESIS, &temp, &dummy))
    {
        return false;
    }

    value = static_cast<int>(temp);

    return true;
}


bool IdsuEye::SetAutoBrightnessHysteresis(int value)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    double temp = static_cast<double>(value);
    double dummy = 0;

    if (IS_SUCCESS != is_SetAutoParameter(m_hCam, IS_SET_AUTO_HYSTERESIS, &temp, &dummy))
    {
        return false;
    }

    return true;
}


bool IdsuEye::SaveParametersInCameraMemory()
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (IS_SUCCESS != is_ParameterSet(m_hCam, IS_PARAMETERSET_CMD_SAVE_EEPROM, NULL, NULL))
    {
        return false;
    }

    return true;
}


bool IdsuEye::LoadParametersFromCameraMemory()
{
    if (NULL == m_hCam)
    {
        return false;
    }

    if (IS_SUCCESS != is_ParameterSet(m_hCam, IS_PARAMETERSET_CMD_LOAD_EEPROM, NULL, NULL))
    {
        return false;
    }

    return true;
}


bool IdsuEye::GetSubsamplingFactor(int& x, int& y)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    x = is_SetSubSampling(m_hCam, IS_GET_SUBSAMPLING_FACTOR_HORIZONTAL);
    y = is_SetSubSampling(m_hCam, IS_GET_SUBSAMPLING_FACTOR_VERTICAL);
    
    return true;
}


bool IdsuEye::SetSubsamplingFactor(int x, int y)
{
    if (NULL == m_hCam)
    {
        return false;
    }

    int factorX = 1;
    int factorY = 1;

    if (1 == x)
    {
        factorX = IS_SUBSAMPLING_DISABLE;
    }
    else if (2 == x)
    {
        factorX = IS_SUBSAMPLING_2X_HORIZONTAL;
    }
    else if (4 == x)
    {
        factorX = IS_SUBSAMPLING_4X_HORIZONTAL;
    }
    else if (8 == x)
    {
        factorX = IS_SUBSAMPLING_8X_HORIZONTAL;
    }
    else if (16 == x)
    {
        factorX = IS_SUBSAMPLING_16X_HORIZONTAL;
    }

    if (1 == y)
    {
        factorY = IS_SUBSAMPLING_DISABLE;
    }
    else if (2 == y)
    {
        factorY = IS_SUBSAMPLING_2X_VERTICAL;
    }
    else if (4 == y)
    {
        factorY = IS_SUBSAMPLING_4X_VERTICAL;
    }
    else if (8 == y)
    {
        factorY = IS_SUBSAMPLING_8X_VERTICAL;
    }
    else if (16 == y)
    {
        factorY = IS_SUBSAMPLING_16X_VERTICAL;
    }

    if (IS_SUCCESS != is_SetSubSampling(m_hCam, factorX | factorY))
    {
        return false;
    }

    return true;
}


/**************************************************************************************/
/* Internal functions                                                                 */
/**************************************************************************************/
UINT IdsuEye::CaptureWorker(LPVOID param)
{
    IdsuEye* parent = (IdsuEye*)param;

    if (parent == NULL)
    {
        return FALSE;
    }

    if (NULL == parent->m_hCam)
    {
        return false;
    }

    parent->m_stopThread = FALSE;
    parent->m_threadRunning = TRUE;

    while (TRUE == parent->m_threadRunning)
    {
        DWORD ret = WaitForSingleObject(parent->m_frameEvent, 5000);
        if (WAIT_OBJECT_0 == ret)
        {
            int dummy   = 0;
            char *memLast = NULL;
            char *mem  = NULL;

            if (IS_SUCCESS == is_GetActSeqBuf(parent->m_hCam, &dummy, &mem, &memLast))
            {
                if (IS_SUCCESS == is_LockSeqBuf(parent->m_hCam, IS_IGNORE_PARAMETER, memLast))
                {
                    int memoryId = 0;
                    for (int i = 0; i < NUMBER_OF_SEQUENCE_BUFFERS; i++)
                    {
                        if (memLast == parent->m_memory[i])
                        {
                            memoryId = parent->m_memoryId[i];
                            break;
                        }
                    }

                    is_RenderBitmap(parent->m_hCam, memoryId, parent->m_hWnd, parent->m_renderType);
                    is_UnlockSeqBuf(parent->m_hCam, IS_IGNORE_PARAMETER, memLast);
                }
            }
        }
        else if (WAIT_TIMEOUT == ret)
        {
        }

        if (TRUE == parent->m_stopThread)
        {
            parent->m_threadRunning = FALSE;
        }
    }

    return TRUE;
}
