//===========================================================================//
//                                                                           //
//  Copyright (C) 2004 - 2019                                                //
//  IDS Imaging Development Systems GmbH                                     //
//  Dimbacherstr. 6-8                                                        //
//  D-74182 Obersulm-Willsbach                                               //
//                                                                           //
//  The information in this document is subject to change without            //
//  notice and should not be construed as a commitment by IDS Imaging GmbH.  //
//  IDS Imaging GmbH does not assume any responsibility for any errors       //
//  that may appear in this document.                                        //
//                                                                           //
//  This document, or source code, is provided solely as an example          //
//  of how to utilize IDS software libraries in a sample application.        //
//  IDS Imaging GmbH does not assume any responsibility for the use or       //
//  reliability of any portion of this document or the described software.   //
//                                                                           //
//  General permission to copy or modify, but not for profit, is hereby      //
//  granted,  provided that the above copyright notice is included and       //
//  reference made to the fact that reproduction privileges were granted     //
//  by IDS Imaging GmbH.                                                     //
//                                                                           //
//  IDS cannot assume any responsibility for the use or misuse of any        //
//  portion of this software for other than its intended diagnostic purpose  //
//  in calibrating and testing IDS manufactured cameras and software.        //
//                                                                           //
//===========================================================================//

#if !defined IDS_UEYE
#define IDS_UEYE

#include "stdafx.h"

#include "uEye.h"

// We include sherpa here to use the defines
#include "IdsSherpa.h"

#include <iostream>


#define NUMBER_OF_SEQUENCE_BUFFERS  10


class IdsuEye
{

public:

    IdsuEye(HWND hWnd);
    ~IdsuEye();

    // Open/Close camera
    bool InitCamera(std::string serialNumber = "");
    bool ExitCamera();

    // Current number of connected cameras
    bool GetNumberOfCameras(int& number);

    // Pixel format
    bool SetColorMode(int colorMode);
    bool GetColorMode(int& colorMode);
    bool GetBitsPerPixel(int colorMode, int& bitsPerPixel);

    // AOI
    bool GetImageAoiMin(int& x, int& y, int& width, int& height);
    bool GetImageAoiMax(int& x, int& y, int& width, int& height);
    bool GetImageAoi(int& x, int& y, int& width, int& height);
    bool SetImageAoi(int x, int y, int width, int height);

    // Information
    bool GetCameraInfo(CAMERA_INFORMATION& info);

    // Memory allocation
    bool AllocImageBuffersAndCreateSequence(int width, int height, int bitsPerPixel);
    bool FreeImageBuffersAndClearSequence();
    
    // Initialization of the display window
    bool SetRenderType(int renderType);

    // Activation/Deactivation of capture events
    bool InitEvents();
    bool ExitEvents();

    // Start/Stop capture
    bool StartCapture();
    bool StopCapture();  
    bool IsCapturing();

    // Framerate
    bool GetFramerateMin(double& min);
    bool GetFramerateMax(double& max);
    bool GetFramerate(double& framerate);
    bool SetFramerate(double framerate);

    // Exposure time
    bool GetExposureTimeMin_ms(double& min);
    bool GetExposureTimeMax_ms(double& max);
    bool GetExposureTime_ms(double& exposure);
    bool SetExposureTime_ms(double exposure);

    // Trigger
    bool GetTriggerMode(int& mode);
    bool SetTriggerMode(int mode);
    bool ForceTrigger();
    bool SetTriggerTimeout_ms(int timeout);
    bool GetTriggerDelayMin_us(int& min);
    bool GetTriggerDelayMax_us(int& max);
    bool GetTriggerDelayInc_us(int& inc);
    bool GetTriggerDelay_us(int& delay);
    bool SetTriggerDelay_us(int delay);

    // Gain
    bool GetGainMin(int type, double& min);
    bool GetGainMax(int type, double& max);
    bool GetGain(int type, double& gain);
    bool SetGain(int type, double gain);

    // Auto exposure
    bool GetAutoExposure(int& autoExposure);
    bool SetAutoExposure(int autoExposure);

    // Auto gain
    bool GetAutoGain(int& autoGain);
    bool SetAutoGain(int autoGain);

    bool GetAutoBrightnessOnce(int& once);
    bool SetAutoBrightnessOnce(int once);

    bool GetAutoBrightnessReference(int& value);
    bool SetAutoBrightnessReference(int value);

    bool GetAutoBrightnessSpeed(int& value);
    bool SetAutoBrightnessSpeed(int value);

    bool GetAutoBrightnessSkipframesMin(int& min);
    bool GetAutoBrightnessSkipframesMax(int& max);
    bool GetAutoBrightnessSkipframes(int& value);
    bool SetAutoBrightnessSkipframes(int value);

    bool GetAutoBrightnessHysteresisMin(int& min);
    bool GetAutoBrightnessHysteresisMax(int& max);
    bool GetAutoBrightnessHysteresis(int& value);
    bool SetAutoBrightnessHysteresis(int value);

    // Parameter sets
    bool SaveParametersInCameraMemory();
    bool LoadParametersFromCameraMemory();

    bool GetSubsamplingFactor(int& x, int& y);
    bool SetSubsamplingFactor(int x, int y);


private:

    HWND m_hWnd;
    HIDS m_hCam;
        
    char* m_memory[NUMBER_OF_SEQUENCE_BUFFERS];
    int m_memoryId[NUMBER_OF_SEQUENCE_BUFFERS];
    
    CWinThread* m_threadCapture;
    BOOL m_threadRunning;
    BOOL m_stopThread;

    HANDLE m_frameEvent;

    int m_renderType;

    static UINT	CaptureWorker(LPVOID pParam);
};

#endif // #define IDS_UEYE
